@@ -1,5 +1,5 @@
 /*
- *     Copyright (c) 2008-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+ *     Copyright (c) 2008-2016 CoNWeT Lab., Universidad Politécnica de Madrid
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -19,47 +19,45 @@
  *
  */
 
-/*global StyledElements, Wirecloud*/
+/* globals StyledElements */
 
-(function () {
+
+(function (utils) {
 
     "use strict";
 
-    var StyledTextArea, oninput, onfocus, onblur;
+    var TextArea, oninput, onfocus, onblur;
 
     oninput = function oninput() {
-        this.events.change.dispatch(this);
+        this.dispatchEvent('change');
     };
 
     onfocus = function onfocus() {
-        this.wrapperElement.classList.add('focus');
-        this.events.focus.dispatch(this);
+        this.dispatchEvent('focus');
     };
 
     onblur = function onblur() {
-        this.wrapperElement.classList.remove('focus');
-        this.events.blur.dispatch(this);
+        this.dispatchEvent('blur');
     };
 
     /**
      * Styled Text Area.
      */
-    StyledTextArea = function StyledTextArea(options) {
+    TextArea = function TextArea(options) {
         var defaultOptions = {
             'initialValue': '',
             'class': ''
         };
-        options = Wirecloud.Utils.merge(defaultOptions, options);
+        options = utils.merge(defaultOptions, options);
 
-        StyledElements.StyledInputElement.call(this, options.initialValue, ['blur', 'change', 'focus']);
-
-        this.wrapperElement = document.createElement("div");
-        this.wrapperElement.className = "styled_text_area";
-        if (options['class'] !== "") {
-            this.wrapperElement.className += " " + options['class'];
-        }
+        StyledElements.InputElement.call(this, options.initialValue, ['blur', 'change', 'focus']);
 
         this.inputElement = document.createElement("textarea");
+        this.wrapperElement = this.inputElement;
+        this.wrapperElement.className = "se-text-area";
+        if (options.class !== "") {
+            this.wrapperElement.className += " " + options.class;
+        }
 
         if (options.name) {
             this.inputElement.setAttribute("name", options.name);
@@ -71,42 +69,40 @@
 
         this.setValue(options.initialValue);
 
-        var div = document.createElement("div");
-        div.appendChild(this.inputElement);
-        this.wrapperElement.appendChild(div);
-
         /* Internal events */
         this._oninput = oninput.bind(this);
         this._onfocus = onfocus.bind(this);
         this._onblur = onblur.bind(this);
 
-        this.inputElement.addEventListener('mousedown', Wirecloud.Utils.stopPropagationListener, true);
-        this.inputElement.addEventListener('click', Wirecloud.Utils.stopPropagationListener, true);
+        this.inputElement.addEventListener('mousedown', utils.stopPropagationListener, true);
+        this.inputElement.addEventListener('click', utils.stopPropagationListener, true);
         this.inputElement.addEventListener('input', this._oninput, true);
         this.inputElement.addEventListener('focus', this._onfocus, true);
         this.inputElement.addEventListener('blur', this._onblur, true);
+        this.inputElement.addEventListener('keydown', utils.stopInputKeydownPropagationListener, false);
     };
-    StyledTextArea.prototype = new StyledElements.StyledInputElement();
+    utils.inherit(TextArea, StyledElements.InputElement);
 
-    StyledTextArea.prototype.select = function select() {
+    TextArea.prototype.select = function select() {
         this.inputElement.select();
     };
 
-    StyledTextArea.prototype.destroy = function destroy() {
+    TextArea.prototype.destroy = function destroy() {
 
-        this.inputElement.removeEventListener('mousedown', Wirecloud.Utils.stopPropagationListener, true);
-        this.inputElement.removeEventListener('click', Wirecloud.Utils.stopPropagationListener, true);
+        this.inputElement.removeEventListener('mousedown', utils.stopPropagationListener, true);
+        this.inputElement.removeEventListener('click', utils.stopPropagationListener, true);
         this.inputElement.removeEventListener('input', this._oninput, true);
         this.inputElement.removeEventListener('focus', this._onfocus, true);
         this.inputElement.removeEventListener('blur', this._onblur, true);
+        this.inputElement.removeEventListener('keydown', utils.stopInputKeydownPropagationListener, false);
 
         delete this._oninput;
         delete this._onfocus;
         delete this._onblur;
 
-        StyledElements.StyledInputElement.prototype.destroy.call(this);
+        StyledElements.InputElement.prototype.destroy.call(this);
     };
 
-    StyledElements.StyledTextArea = StyledTextArea;
+    StyledElements.TextArea = TextArea;
 
-})();
+})(StyledElements.Utils);
