@@ -1,5 +1,5 @@
 /*
- *     Copyright 2013-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+ *     Copyright 2013-2016 (c) CoNWeT Lab., Universidad Politécnica de Madrid
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -19,30 +19,31 @@
  *
  */
 
-/*global gettext, StyledElements, Wirecloud*/
+/* globals StyledElements, Wirecloud */
 
-(function () {
+
+(function (se, utils) {
 
     "use strict";
 
     var setDialogPosition = function setDialogPosition(ref_pos, pos) {
         switch (pos) {
-            case 'up':
-                this.htmlElement.style.top = (ref_pos.top - this.htmlElement.offsetHeight - 20) + 'px';
-                this.htmlElement.style.left = (ref_pos.left + (ref_pos.width - this.htmlElement.offsetWidth) / 2) + 'px';
-                break;
-            case 'right':
-                this.htmlElement.style.top = (ref_pos.top + (ref_pos.height - this.htmlElement.offsetHeight) / 2) + 'px';
-                this.htmlElement.style.left = (ref_pos.right + 20) + 'px';
-                break;
-            case 'left':
-                this.htmlElement.style.top = (ref_pos.top + (ref_pos.height - this.htmlElement.offsetHeight) / 2) + 'px';
-                this.htmlElement.style.left = (ref_pos.left - this.htmlElement.offsetWidth - 20) + 'px';
-                break;
-            case 'down':
-                this.htmlElement.style.top = (ref_pos.bottom + 20) + 'px';
-                this.htmlElement.style.left = (ref_pos.left + (ref_pos.width - this.htmlElement.offsetWidth) / 2) + 'px';
-                break;
+        case 'up':
+            this.htmlElement.style.top = (ref_pos.top - this.htmlElement.offsetHeight - 20) + 'px';
+            this.htmlElement.style.left = (ref_pos.left + (ref_pos.width - this.htmlElement.offsetWidth) / 2) + 'px';
+            break;
+        case 'right':
+            this.htmlElement.style.top = (ref_pos.top + (ref_pos.height - this.htmlElement.offsetHeight) / 2) + 'px';
+            this.htmlElement.style.left = (ref_pos.right + 20) + 'px';
+            break;
+        case 'left':
+            this.htmlElement.style.top = (ref_pos.top + (ref_pos.height - this.htmlElement.offsetHeight) / 2) + 'px';
+            this.htmlElement.style.left = (ref_pos.left - this.htmlElement.offsetWidth - 20) + 'px';
+            break;
+        case 'down':
+            this.htmlElement.style.top = (ref_pos.bottom + 20) + 'px';
+            this.htmlElement.style.left = (ref_pos.left + (ref_pos.width - this.htmlElement.offsetWidth) / 2) + 'px';
+            break;
         }
     };
 
@@ -79,7 +80,7 @@
         this.last = false;
         this.pos = options.pos;
         this.title = options.title;
-        this.nextButtonText = gettext('Next');
+        this.nextButtonText = utils.gettext('Next');
         if (options.nextButtonText) {
             this.nextButtonText = options.nextButtonText;
         }
@@ -88,19 +89,17 @@
 
         this.windowContent.innerHTML = options.msg;
 
-        this.nextButton = new StyledElements.StyledButton({
-            'title': gettext("next"),
+        this.nextButton = new se.Button({
             'class': 'nextButton btn-primary',
-            'text': gettext(this.nextButtonText),
+            'text': utils.gettext(this.nextButtonText),
         });
 
         this.nextButton.insertInto(this.windowBottom);
 
         // Cancel button
-        this.cancelButton = new StyledElements.StyledButton({
-            'title': gettext("Cancel"),
+        this.cancelButton = new se.Button({
             'class': 'cancelButton',
-            'text': gettext("Cancel"),
+            'text': utils.gettext("Cancel"),
         });
         this.cancelButton.insertInto(this.windowBottom);
         this.cancelButton.addEventListener('click', this._closeListener);
@@ -109,7 +108,7 @@
 
         this.wrapperElement = this.htmlElement;
     };
-    SimpleDescription.prototype = new Wirecloud.ui.WindowMenu();
+    utils.inherit(SimpleDescription, Wirecloud.ui.WindowMenu);
 
     SimpleDescription.prototype._closeListener = function _closeListener(e) {
         Wirecloud.ui.WindowMenu.prototype._closeListener.call(this, e);
@@ -141,11 +140,11 @@
      */
     SimpleDescription.prototype.setLast = function setLast(buttonLabel, optionalHandler) {
         this.last = true;
-        Wirecloud.Utils.removeFromParent(this.nextButton.wrapperElement);
+        this.nextButton.remove();
         if (buttonLabel == null) {
-            buttonLabel = gettext('Close');
+            buttonLabel = utils.gettext('Close');
         }
-        this.cancelButton.setLabel(gettext(buttonLabel));
+        this.cancelButton.setLabel(utils.gettext(buttonLabel));
         if (optionalHandler != null) {
             this.cancelButton.removeEventListener('click', this._closeListener);
             this.cancelButton.addEventListener('click', optionalHandler.bind(this));
@@ -157,8 +156,8 @@
      */
     SimpleDescription.prototype.setNext = function setNext() {
         this.nextButton.addEventListener('click', function () {
-            if (this.element != null) {
-                this.element.classList.remove('tuto_highlight');
+            if (this.currentElement != null) {
+                this.currentElement.classList.remove('tuto_highlight');
             }
             this.tutorial.nextStep();
         }.bind(this));
@@ -172,14 +171,14 @@
 
         this.htmlElement.classList.add("activeStep");
         if (typeof this.element === 'function') {
-            this.element = this.element();
-            this.tutorial.setControlLayer(this.element, true);
-            this.element.classList.add('tuto_highlight');
+            this.currentElement = this.element();
+            this.tutorial.setControlLayer(this.currentElement, true);
+            this.currentElement.classList.add('tuto_highlight');
         } else {
             this.tutorial.resetControlLayer(false);
         }
-        if (this.element != null) {
-            ref_pos = this.element.getBoundingClientRect();
+        if (this.currentElement != null) {
+            ref_pos = this.currentElement.getBoundingClientRect();
             weights = [];
             positions = ['down', 'left', 'top', 'right'];
             do {
@@ -199,7 +198,7 @@
         this.nextButton.focus();
     };
 
-     /**
+    /**
      * activate this step
      */
     SimpleDescription.prototype.activate = function activate() {
@@ -210,9 +209,6 @@
         }
     };
 
-    /*************************************************************************
-     * Make SimpleDescription public
-     *************************************************************************/
     Wirecloud.ui.Tutorial.SimpleDescription = SimpleDescription;
 
-})();
+})(StyledElements, Wirecloud.Utils);
