@@ -1,5 +1,5 @@
 /*
- *     (C) Copyright 2012 Universidad Politécnica de Madrid
+ *     Copyright (c) 2012-2016 CoNWeT Lab., Universidad Politécnica de Madrid
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -19,7 +19,8 @@
  *
  */
 
-/*global Wirecloud*/
+/* globals NodeList, Wirecloud */
+
 
 (function () {
 
@@ -59,7 +60,7 @@
     };
 
     var configure_next_step_phase = function configure_next_step_phase() {
-        var msg;
+        var msg, targetElement;
 
         if (this.popup) {
             this.popup.destroy();
@@ -81,11 +82,17 @@
         }
 
         if (msg) {
-            this.popup = new Wirecloud.ui.Tutorial.PopUp(this.next_element, {
+            if (this.options.targetElement != null) {
+                targetElement = get_simple_element(this.options.targetElement);
+            } else {
+                targetElement = this.next_element;
+            }
+            this.popup = new Wirecloud.ui.Tutorial.PopUp(targetElement, {
                 highlight: true,
+                user: true,
                 msg: msg,
-                position: this.position,
-                closable: !this.withoutCloseButton
+                position: this.nextPosition,
+                closable: true
             });
             this.layer.appendChild(this.popup.wrapperElement);
             this.popup.repaint();
@@ -105,9 +112,10 @@
         this.tutorial.setControlLayer(this.start_element);
         this.popup = new Wirecloud.ui.Tutorial.PopUp(this.start_element, {
             highlight: true,
+            user: true,
             msg: this.options.msg,
             position: this.position,
-            closable: !this.withoutCloseButton
+            closable: true
         });
         this.layer.appendChild(this.popup.wrapperElement);
         this.popup.repaint();
@@ -130,7 +138,7 @@
         this.next_element.removeEventListener(this.event, this.nextHandler, this.options.eventCapture);
         this.next_element = null;
 
-        for (i = 0; i < this.disableLayer.length; i ++) {
+        for (i = 0; i < this.disableLayer.length; i++) {
             this.layer.removeChild(this.disableLayer[i]);
         }
         this.disableLayer = [];
@@ -139,9 +147,6 @@
         configure_start_phase.call(this);
     };
 
-    /*************************************************************************
-     * Constructor
-     *************************************************************************/
     var UserAction = function UserAction(tutorial, options) {
         this.options = options;
         // Normalize asynchronous option
@@ -151,6 +156,7 @@
         this.tutorial = tutorial;
         this.element = options.elem;
         this.position = options.pos;
+        this.nextPosition = options.secondPos ? options.secondPos : this.position;
         if (options.event) {
             this.event = options.event;
         } else {
@@ -166,7 +172,7 @@
         this.restart_handlers = [];
         this.disableLayer = [];
         if (options.eventCapture != null) {
-            options.eventCapture = !! options.eventCapture;
+            options.eventCapture = !!options.eventCapture;
         } else {
             options.eventCapture = true;
         }
@@ -198,7 +204,7 @@
      * set next handler
      */
     UserAction.prototype.setNext = function setNext() {
-        this.nextHandler = function (e) {
+        this.nextHandler = function () {
             if (this.options.eventFilterFunction.apply(null, arguments)) {
                 nextHandler.call(this);
             }
@@ -223,7 +229,7 @@
         }
 
         var disableElems = build_disable_elements_list(this.disableElems);
-        for (i = 0; i < disableElems.length; i ++) {
+        for (i = 0; i < disableElems.length; i++) {
             this.disableLayer[i] = this.disable(disableElems[i]);
         }
 
@@ -238,7 +244,7 @@
 
         pos = elem.getBoundingClientRect();
         disableLayer = document.createElement("div");
-        disableLayer.addClassName('disableLayer');
+        disableLayer.className = 'disableLayer';
         disableLayer.style.top = pos.top + 'px';
         disableLayer.style.left = pos.left + 'px';
         disableLayer.style.width = pos.width + 'px';
@@ -260,11 +266,10 @@
         this.tutorial.nextStep();
     };
 
-    var _activate = function _activate(element, withoutCloseButton) {
+    var _activate = function _activate(element) {
         if (element != null) {
             this.start_element = element;
         }
-        this.withoutCloseButton = withoutCloseButton;
 
         if (this.options.eventToDeactivateLayer != null) {
             configure_start_phase.call(this);
@@ -276,11 +281,11 @@
     /**
      * activate this step
      */
-    UserAction.prototype.activate = function activate(withoutCloseButton) {
+    UserAction.prototype.activate = function activate() {
         if (this.options.asynchronous) {
             this.element(_activate.bind(this));
         } else {
-            _activate.call(this, null, withoutCloseButton);
+            _activate.call(this, null);
         }
     };
 
@@ -300,7 +305,7 @@
             this.next_element = null;
         }
 
-        for (i = 0; i < this.disableLayer.length; i ++) {
+        for (i = 0; i < this.disableLayer.length; i++) {
             this.layer.removeChild(this.disableLayer[i]);
         }
         this.disableLayer = null;
@@ -314,8 +319,6 @@
         this.arrow = null;
     };
 
-    /*************************************************************************
-     * Make Anchor public
-     *************************************************************************/
     Wirecloud.ui.Tutorial.UserAction = UserAction;
+
 })();
