@@ -31,11 +31,12 @@ from mock import DEFAULT, patch, Mock, ANY
 
 from wirecloud.commons.exceptions import ErrorResponse
 from wirecloud.commons.utils.html import clean_html, filter_changelog
-from wirecloud.commons.utils.http import build_downloadfile_response, build_sendfile_response, get_current_domain, get_current_scheme, get_content_type, normalize_boolean_param, produces, validate_url_param
+from wirecloud.commons.utils.http import build_downloadfile_response, build_sendfile_response, get_absolute_static_url, get_current_domain, get_current_scheme, get_content_type, normalize_boolean_param, produces, validate_url_param
 from wirecloud.commons.utils.log import SkipUnreadablePosts
 from wirecloud.commons.utils.mimeparser import best_match, InvalidMimeType, parse_mime_type
 from wirecloud.commons.utils.version import Version
 from wirecloud.commons.utils.wgt import WgtFile
+from wirecloud.platform.core.plugins import get_version_hash
 
 
 # Avoid nose to repeat these tests (they are run through wirecloud/commons/tests/__init__.py)
@@ -387,12 +388,6 @@ class HTTPUtilsTestCase(TestCase):
 
     tags = ('wirecloud-utils', 'wirecloud-general-utils', 'wirecloud-http-utils', 'wirecloud-noselenium')
 
-    @classmethod
-    def setUpClass(cls):
-
-        super(HTTPUtilsTestCase, cls).setUpClass()
-        cls.get_current_site_import = 'django.contrib.sites.shortcuts.get_current_site' if django.VERSION[1] >= 7 else 'django.contrib.sites.models.get_current_site'
-
     def _prepare_request_mock(self):
 
         request = Mock()
@@ -571,6 +566,31 @@ class HTTPUtilsTestCase(TestCase):
         request.META['CONTENT_TYPE'] = 'application/json/ji'
         self.assertEqual(get_content_type(request), ('', {}))
 
+    @override_settings(STATIC_URL="/static/")
+    @patch("wirecloud.commons.utils.http.get_current_scheme")
+    @patch("wirecloud.commons.utils.http.get_current_domain")
+    def test_get_absolute_static_url_unversioned(self, get_current_domain, get_current_scheme):
+        request = self._prepare_request_mock()
+        get_current_scheme.return_value = "http"
+        get_current_domain.return_value = "dashboards.example.org"
+
+        self.assertEqual(get_absolute_static_url("/path", request=request), 'http://dashboards.example.org/path')
+
+        get_current_scheme.assert_called_once_with(request)
+        get_current_domain.assert_called_once_with(request)
+
+    @override_settings(STATIC_URL="/static/")
+    @patch("wirecloud.commons.utils.http.get_current_scheme")
+    @patch("wirecloud.commons.utils.http.get_current_domain")
+    def test_get_absolute_static_url_versioned(self, get_current_domain, get_current_scheme):
+        request = self._prepare_request_mock()
+        get_current_scheme.return_value = "http"
+        get_current_domain.return_value = "dashboards.example.org"
+        self.assertEqual(get_absolute_static_url("/path", request=request, versioned=True), 'http://dashboards.example.org/path?v=' + get_version_hash())
+
+        get_current_scheme.assert_called_once_with(request)
+        get_current_domain.assert_called_once_with(request)
+
     @override_settings(FORCE_PROTO=None)
     def test_get_current_scheme_http(self):
         request = self._prepare_request_mock()
@@ -602,8 +622,8 @@ class HTTPUtilsTestCase(TestCase):
     @override_settings(FORCE_PROTO=None, FORCE_DOMAIN=None, FORCE_PORT=None)
     def test_get_current_domain(self):
         request = self._prepare_request_mock()
-        with patch(self.get_current_site_import) as get_current_site_mock:
-            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT) as mocks:
+        with patch('django.contrib.sites.shortcuts.get_current_site') as get_current_site_mock:
+            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT, _servername=None) as mocks:
                 get_current_site_mock.return_value = self._prepare_site_mock()
                 mocks['get_current_scheme'].return_value = 'http'
                 self.assertEqual(get_current_domain(request), 'site.example.com:8000')
@@ -612,8 +632,8 @@ class HTTPUtilsTestCase(TestCase):
     @override_settings(FORCE_PROTO=None, FORCE_DOMAIN='myserver.com', FORCE_PORT=8080)
     def test_get_current_domain_forced(self):
         request = self._prepare_request_mock()
-        with patch(self.get_current_site_import) as get_current_site_mock:
-            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT) as mocks:
+        with patch('django.contrib.sites.shortcuts.get_current_site') as get_current_site_mock:
+            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT, _servername=None) as mocks:
                 mocks['get_current_scheme'].return_value = 'http'
                 self.assertEqual(get_current_domain(request), 'myserver.com:8080')
                 self.assertEqual(mocks['socket'].getfqdn.call_count, 0)
@@ -622,8 +642,8 @@ class HTTPUtilsTestCase(TestCase):
     @override_settings(FORCE_PROTO=None, FORCE_DOMAIN='forced.example.com', FORCE_PORT=8000)
     def test_get_current_domain_forced_domain(self):
         request = self._prepare_request_mock()
-        with patch(self.get_current_site_import) as get_current_site_mock:
-            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT) as mocks:
+        with patch('django.contrib.sites.shortcuts.get_current_site') as get_current_site_mock:
+            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT, _servername=None) as mocks:
                 get_current_site_mock.return_value = self._prepare_site_mock()
                 mocks['get_current_scheme'].return_value = 'http'
                 self.assertEqual(get_current_domain(request), 'forced.example.com:8000')
@@ -632,8 +652,8 @@ class HTTPUtilsTestCase(TestCase):
     @override_settings(FORCE_PROTO=None, FORCE_DOMAIN=None, FORCE_PORT=81)
     def test_get_current_domain_forced_port(self):
         request = self._prepare_request_mock()
-        with patch(self.get_current_site_import) as get_current_site_mock:
-            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT) as mocks:
+        with patch('django.contrib.sites.shortcuts.get_current_site') as get_current_site_mock:
+            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT, _servername=None) as mocks:
                 get_current_site_mock.return_value = self._prepare_site_mock()
                 mocks['get_current_scheme'].return_value = 'http'
                 self.assertEqual(get_current_domain(request), 'site.example.com:81')
@@ -642,8 +662,8 @@ class HTTPUtilsTestCase(TestCase):
     @override_settings(FORCE_PROTO=None, FORCE_DOMAIN=None, FORCE_PORT=80)
     def test_get_current_domain_fallback_http(self):
         request = self._prepare_request_mock()
-        with patch(self.get_current_site_import) as get_current_site_mock:
-            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT) as mocks:
+        with patch('django.contrib.sites.shortcuts.get_current_site') as get_current_site_mock:
+            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT, _servername=None) as mocks:
                 get_current_site_mock.side_effect = Exception
                 mocks['socket'].getfqdn.return_value = 'fqdn.example.com'
                 mocks['get_current_scheme'].return_value = 'http'
@@ -652,8 +672,8 @@ class HTTPUtilsTestCase(TestCase):
     @override_settings(FORCE_PROTO=None, FORCE_DOMAIN=None, FORCE_PORT=81)
     def test_get_current_domain_fallback_http_custom_port(self):
         request = self._prepare_request_mock()
-        with patch(self.get_current_site_import) as get_current_site_mock:
-            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT) as mocks:
+        with patch('django.contrib.sites.shortcuts.get_current_site') as get_current_site_mock:
+            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT, _servername=None) as mocks:
                 get_current_site_mock.side_effect = Exception
                 mocks['socket'].getfqdn.return_value = 'fqdn.example.com'
                 mocks['get_current_scheme'].return_value = 'http'
@@ -662,8 +682,8 @@ class HTTPUtilsTestCase(TestCase):
     @override_settings(FORCE_DOMAIN=None, FORCE_PORT=443)
     def test_get_current_domain_fallback_https(self):
         request = self._prepare_request_mock()
-        with patch(self.get_current_site_import) as get_current_site_mock:
-            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT) as mocks:
+        with patch('django.contrib.sites.shortcuts.get_current_site') as get_current_site_mock:
+            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT, _servername=None) as mocks:
                 get_current_site_mock.side_effect = Exception
                 mocks['socket'].getfqdn.return_value = 'fqdn.example.com'
                 mocks['get_current_scheme'].return_value = 'https'
@@ -672,9 +692,9 @@ class HTTPUtilsTestCase(TestCase):
     @override_settings(FORCE_DOMAIN=None, FORCE_PORT=8443)
     def test_get_current_domain_fallback_https_custom_port(self):
         request = self._prepare_request_mock()
-        with patch(self.get_current_site_import) as get_current_site_mock:
-            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT) as mocks:
+        with patch('django.contrib.sites.shortcuts.get_current_site') as get_current_site_mock:
+            with patch.multiple('wirecloud.commons.utils.http', socket=DEFAULT, get_current_scheme=DEFAULT, _servername=None) as mocks:
                 get_current_site_mock.side_effect = Exception
                 mocks['socket'].getfqdn.return_value = 'example.com'
                 mocks['get_current_scheme'].return_value = 'http'
-                self.assertEqual(get_current_domain(request), 'fqdn.example.com:8443')
+                self.assertEqual(get_current_domain(request), 'example.com:8443')
