@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2012-2013 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2012-2017 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -23,9 +23,11 @@ import rdflib
 
 from django.utils.translation import ugettext as _
 from lxml import etree
+from six import text_type
 
+from wirecloud.commons.utils.mimeparser import parse_mime_type
 from wirecloud.commons.utils.template.base import is_valid_name, is_valid_vendor, is_valid_version, TemplateParseException
-from wirecloud.commons.utils.http import parse_mime_type
+from wirecloud.platform.wiring.utils import parse_wiring_old_version, get_wiring_skeleton
 
 # Namespaces used by rdflib
 WIRE = rdflib.Namespace("http://wirecloud.conwet.fi.upm.es/ns/widget#")
@@ -38,6 +40,7 @@ RDF = rdflib.Namespace(RDF_NS)
 RDFS = rdflib.Namespace("http://www.w3.org/2000/01/rdf-schema#")
 VCARD = rdflib.Namespace("http://www.w3.org/2006/vcard/ns#")
 BLUEPRINT = rdflib.Namespace("http://bizweb.sap.com/TR/blueprint#")
+DOAP = rdflib.Namespace('http://usefulinc.com/ns/doap#')
 
 
 def possible_int(value):
@@ -62,9 +65,9 @@ class RDFTemplateParser(object):
                 self._graph = rdflib.Graph()
                 self._graph.parse(data=template, format='n3')
             except:
-                if isinstance(template, str):
+                if isinstance(template, bytes):
                     doc = etree.fromstring(template)
-                elif isinstance(template, unicode):
+                elif isinstance(template, text_type):
                     # Work around: ValueError: Unicode strings with encoding
                     # declaration are not supported.
                     doc = etree.fromstring(template.encode('utf-8'))
@@ -72,13 +75,13 @@ class RDFTemplateParser(object):
                 root_element_qname = etree.QName(doc)
 
                 if root_element_qname.namespace is None:
-                    raise TemplateParseException("XML document does not contain a valid rdf namespace")
+                    raise ValueError("XML document does not contain a valid rdf namespace")
 
                 if root_element_qname.namespace != RDF_NS:
-                    raise TemplateParseException("Invalid namespace: " + root_element_qname.namespace)
+                    raise ValueError("Invalid namespace: " + root_element_qname.namespace)
 
                 if root_element_qname.localname != 'RDF':
-                    raise TemplateParseException("Invalid root element: " + root_element_qname.localname)
+                    raise ValueError("Invalid root element: " + root_element_qname.localname)
 
                 self._graph = rdflib.Graph()
                 self._graph.parse(data=template, format='xml')
@@ -102,15 +105,15 @@ class RDFTemplateParser(object):
                     self._info['type'] = 'mashup'
                     break
                 else:
-                    raise TemplateParseException('RDF document does not describe a widget, operator or mashup resource')
+                    raise TemplateParseException('RDF document does not describe a widget, operator or mashup component')
 
         self._parse_basic_info()
 
     def _add_translation_index(self, value, **kwargs):
 
         if value not in self._translation_indexes:
-            self._translation_indexes[value] = []
-            self._translation_indexes[value].append(kwargs)
+            self._translation_indexes[text_type(value)] = []
+            self._translation_indexes[text_type(value)].append(kwargs)
 
     def _get_translation_field(self, namespace, element, subject, translation_name, required=True, **kwargs):
 
@@ -118,15 +121,20 @@ class RDFTemplateParser(object):
         base_value = None
 
         for field_element in self._graph.objects(subject, namespace[element]):
+
+            if not isinstance(field_element, rdflib.Literal):
+                msg = _('invalid content for field: %(field)s')
+                raise TemplateParseException(msg % {'field': element})
+
             if field_element.language:
                 translated = True
 
                 if field_element.language not in self._translations:
-                    self._translations[unicode(field_element.language)] = {}
+                    self._translations[text_type(field_element.language)] = {}
 
-                self._translations[unicode(field_element.language)][translation_name] = unicode(field_element)
+                self._translations[text_type(field_element.language)][translation_name] = text_type(field_element)
             else:
-                base_value = unicode(field_element)
+                base_value = text_type(field_element)
 
         if base_value is not None and translated is True:
             if 'en' not in self._translations:
@@ -156,7 +164,7 @@ class RDFTemplateParser(object):
         fields = self._graph.objects(subject, namespace[element])
         for field_element in fields:
             if not id_:
-                result = unicode(field_element)
+                result = text_type(field_element)
                 break
             else:
                 result = field_element
@@ -169,10 +177,33 @@ class RDFTemplateParser(object):
                 result = default
         return result
 
+    def _parse_people_field(self, namespace, element, subject):
+
+        people = []
+        sorted_people = sorted(self._graph.objects(subject, namespace[element]), key=lambda person: possible_int(self._get_field(WIRE, 'index', person, required=False)))
+        for person in sorted_people:
+            name = self._get_field(FOAF, 'name', person, required=False)
+            if name == '':
+                continue
+
+            person_info = {'name': name}
+
+            email = self._get_field(FOAF, 'mbox', person, required=False)
+            if email != '':
+                person_info['email'] = email
+
+            homepage = self._get_field(FOAF, 'homepage', person, required=False)
+            if homepage != '':
+                person_info['url'] = homepage
+
+            people.append(person_info)
+
+        return people
+
     def _parse_extra_info(self):
 
         if self._info['type'] == 'widget' or self._info['type'] == 'operator':
-            self._parse_widget_info()
+            self._parse_component_info()
         elif self._info['type'] == 'mashup':
             self._parse_workspace_info()
 
@@ -186,74 +217,89 @@ class RDFTemplateParser(object):
 
         # ------------------------------------------
         if self._info['type'] == 'widget':
-            self._rootURI = self._graph.subjects(RDF['type'], WIRE['Widget']).next()
+            self._rootURI = next(self._graph.subjects(RDF['type'], WIRE['Widget']))
         elif self._info['type'] == 'mashup':
-            self._rootURI = self._graph.subjects(RDF['type'], WIRE_M['Mashup']).next()
+            self._rootURI = next(self._graph.subjects(RDF['type'], WIRE_M['Mashup']))
         elif self._info['type'] == 'operator':
-            self._rootURI = self._graph.subjects(RDF['type'], WIRE['Operator']).next()
+            self._rootURI = next(self._graph.subjects(RDF['type'], WIRE['Operator']))
 
         vendor = self._get_field(USDL, 'hasProvider', self._rootURI, id_=True)
         self._info['vendor'] = self._get_field(FOAF, 'name', vendor)
         if not is_valid_vendor(self._info['vendor']):
-            raise TemplateParseException(_('ERROR: the format of the vendor is invalid.'))
+            raise TemplateParseException(_('The format of the vendor is invalid.'))
 
         self._info['name'] = self._get_field(DCTERMS, 'title', self._rootURI)
         if not is_valid_name(self._info['name']):
-            raise TemplateParseException(_('ERROR: the format of the name is invalid.'))
+            raise TemplateParseException(_('The format of the name is invalid.'))
 
         self._info['version'] = self._get_field(USDL, 'versionInfo', self._rootURI)
         if not is_valid_version(self._info['version']):
-            raise TemplateParseException(_('ERROR: the format of the version number is invalid. Format: X.X.X where X is an integer. Ex. "0.1", "1.11" NOTE: "1.01" should be changed to "1.0.1" or "1.1"'))
+            raise TemplateParseException(_('The format of the version number is invalid. Format: X.X.X where X is an integer. Ex. "0.1", "1.11" NOTE: "1.01" should be changed to "1.0.1" or "1.1"'))
 
         license = self._get_field(DCTERMS, 'license', self._rootURI, required=False, default=None, id_=True)
         if license is not None:
-            self._info['licenseurl'] = unicode(license)
+            self._info['licenseurl'] = text_type(license)
             self._info['license'] = self._get_field(RDFS, 'label', license, required=False)
         else:
             self._info['licenseurl'] = ''
             self._info['license'] = ''
 
-        self._info['description'] = self._get_translation_field(DCTERMS, 'description', self._rootURI, 'description', type='resource', field='description')
+        longdescription = self._get_field(DCTERMS, 'description', self._rootURI, id_=True, required=False)
+        if longdescription != '' and isinstance(longdescription, rdflib.Literal):
+            # Old and deprecated behaviour
+            self._info['description'] = self._get_translation_field(DCTERMS, 'description', self._rootURI, 'description', required=False, type='resource', field='description')
+            self._info['longdescription'] = ''
+        else:
+            self._info['longdescription'] = '%s' % longdescription
+            self._info['description'] = self._get_translation_field(DCTERMS, 'abstract', self._rootURI, 'description', required=False, type='resource', field='description')
 
-        authors = self._get_field(DCTERMS, 'creator', self._rootURI, required=False, id_=True)
-        self._info['authors'] = self._get_field(FOAF, 'name', authors, required=False)
+        self._info['authors'] = self._parse_people_field(DCTERMS, 'creator', self._rootURI)
+        self._info['contributors'] = self._parse_people_field(DCTERMS, 'contributor', self._rootURI)
 
         self._info['image'] = self._get_field(WIRE, 'hasImageUri', self._rootURI, required=False)
         self._info['smartphoneimage'] = self._get_field(WIRE, 'hasiPhoneImageUri', self._rootURI, required=False)
 
+        self._info['changelog'] = self._get_field(WIRE, 'hasChangeLog', self._rootURI, required=False)
+        self._info['homepage'] = self._get_field(FOAF, 'homepage', self._rootURI, required=False)
+        self._info['issuetracker'] = self._get_field(DOAP, 'bug-database', self._rootURI, required=False)
         self._info['doc'] = self._get_field(FOAF, 'page', self._rootURI, required=False)
 
         self._info['title'] = self._get_translation_field(WIRE, 'displayName', self._rootURI, 'title', required=False, type='resource', field='title')
 
-        addr_element = self._get_field(VCARD, 'addr', self._rootURI, id_=True)
-        self._info['email'] = self._get_field(VCARD, 'email', addr_element)
+        self._info['email'] = ''
+        addr_element = self._get_field(VCARD, 'addr', self._rootURI, id_=True, default=None, required=False)
+        if addr_element is not None:
+            self._info['email'] = self._get_field(VCARD, 'email', addr_element, required=False)
+
         self._parse_requirements()
 
     def _parse_requirements(self):
         self._info['requirements'] = []
 
         for wrequirement in self._graph.objects(self._rootURI, WIRE['hasRequirement']):
-            if self._graph.objects(wrequirement, RDF['type']).next() == WIRE['Feature']:
+            if next(self._graph.objects(wrequirement, RDF['type'])) == WIRE['Feature']:
                 self._info['requirements'].append({
                     'type': 'feature',
                     'name': self._get_field(RDFS, 'label', wrequirement, required=True),
                 })
 
-    def _parse_wiring_info(self, wiring_property='hasPlatformWiring', parse_connections=False):
+    def _parse_wiring_info(self, wiring_property='hasPlatformWiring'):
 
-        self._info['wiring'] = {
-            'inputs': [],
-            'outputs': [],
-        }
+        if self._info['type'] == 'mashup':
+            self._info['wiring'] = get_wiring_skeleton()
+        else:
+            self._info['wiring'] = {}
+
+        self._info['wiring']['inputs'] = []
+        self._info['wiring']['outputs'] = []
 
         # method self._graph.objects always returns an iterable object not subscriptable,
         # althought only exits one instance
-        wiring_type = WIRE
+        wiring_type = WIRE_M if self._info['type'] == 'mashup' else WIRE
+        wiring_element = self._get_field(wiring_type, wiring_property, self._rootURI, id_=True, required=False)
 
         if self._info['type'] == 'mashup':
-            wiring_type = WIRE_M
-
-        wiring_element = self._get_field(wiring_type, wiring_property, self._rootURI, id_=True, required=False)
+            self._info['wiring']['version'] = self._get_field(USDL, 'versionInfo', wiring_element, default="1.0", required=False)
 
         sorted_inputs = sorted(self._graph.objects(wiring_element, WIRE['hasInputEndpoint']), key=lambda source: possible_int(self._get_field(WIRE, 'index', source, required=False)))
 
@@ -261,10 +307,10 @@ class RDFTemplateParser(object):
             var_name = self._get_field(DCTERMS, 'title', input_endpoint, required=True)
             self._info['wiring']['inputs'].append({
                 'name': var_name,
-                'type': self._get_field(WIRE, 'type', input_endpoint, required=False),
-                'label': self._get_translation_field(RDFS, 'label', input_endpoint, var_name + '_label', required=False, type='vdef', variable=var_name, field='label'),
-                'description': self._get_translation_field(DCTERMS, 'description', input_endpoint, var_name + '_description', required=False, type='vdef', variable=var_name, field='description'),
-                'actionlabel': self._get_translation_field(WIRE, 'inputActionLabel', input_endpoint, var_name + '_actionlabel', required=False, type='vdef', variable=var_name, field='actionlabel'),
+                'type': self._get_field(WIRE, 'type', input_endpoint, required=True),
+                'label': self._get_translation_field(RDFS, 'label', input_endpoint, var_name + '_label', required=False, type='inputendpoint', variable=var_name, field='label'),
+                'description': self._get_translation_field(DCTERMS, 'description', input_endpoint, var_name + '_description', required=False, type='inputendpoint', variable=var_name, field='description'),
+                'actionlabel': self._get_translation_field(WIRE, 'inputActionLabel', input_endpoint, var_name + '_actionlabel', required=False, type='inputendpoint', variable=var_name, field='actionlabel'),
                 'friendcode': self._get_field(WIRE, 'friendcode', input_endpoint, required=False),
             })
 
@@ -274,16 +320,20 @@ class RDFTemplateParser(object):
             var_name = self._get_field(DCTERMS, 'title', output_endpoint, required=True)
             self._info['wiring']['outputs'].append({
                 'name': var_name,
-                'type': self._get_field(WIRE, 'type', output_endpoint, required=False),
-                'label': self._get_translation_field(RDFS, 'label', output_endpoint, var_name + '_label', required=False, type='vdef', variable=var_name, field='label'),
-                'description': self._get_translation_field(DCTERMS, 'description', output_endpoint, var_name + '_description', required=False, type='vdef', variable=var_name, field='description'),
+                'type': self._get_field(WIRE, 'type', output_endpoint, required=True),
+                'label': self._get_translation_field(RDFS, 'label', output_endpoint, var_name + '_label', required=False, type='outputendpoint', variable=var_name, field='label'),
+                'description': self._get_translation_field(DCTERMS, 'description', output_endpoint, var_name + '_description', required=False, type='outputendpoint', variable=var_name, field='description'),
                 'friendcode': self._get_field(WIRE, 'friendcode', output_endpoint, required=False),
             })
 
-        if parse_connections:
+        if self._info['type'] == 'mashup':
             self._parse_wiring_connection_info(wiring_element)
             self._parse_wiring_operator_info(wiring_element)
-            self._parse_wiring_views(wiring_element)
+
+            if self._info['wiring']['version'] == '1.0':
+                self._parse_wiring_views(wiring_element)
+            else:
+                self._parse_wiring_behaviours(wiring_element)
 
     def _parse_wiring_connection_info(self, wiring_element):
 
@@ -295,6 +345,7 @@ class RDFTemplateParser(object):
                 'source': {},
                 'target': {},
             }
+
             for source in self._graph.objects(connection, WIRE_M['hasSource']):
                 connection_info['source'] = {
                     'id': self._get_field(WIRE_M, 'sourceId', source),
@@ -334,20 +385,116 @@ class RDFTemplateParser(object):
                 operator_info['preferences'][self._get_field(DCTERMS, 'title', pref)] = {
                     'readonly': self._get_field(WIRE_M, 'readonly', pref, required=False).lower() == 'true',
                     'hidden': self._get_field(WIRE_M, 'hidden', pref, required=False).lower() == 'true',
-                    'value': self._get_field(WIRE, 'value', pref, required=False),
+                    'value': self._get_field(WIRE, 'value', pref, default=None, required=False),
                 }
 
             self._info['wiring']['operators'][operator_info['id']] = operator_info
 
+    def _parse_wiring_components(self, element, behaviour):
+
+        behaviour['components'] = {
+            'operator': {},
+            'widget': {}
+        }
+
+        for entity_view in self._graph.objects(element, WIRE_M['hasComponentView']):
+
+            type_ = self._get_field(WIRE, 'type', entity_view)
+            id_ = self._get_field(WIRE, 'id', entity_view)
+
+            component_view_description = behaviour['components'][type_][id_] = {}
+
+            component_view_description['collapsed'] = self._get_field(WIRE_M, 'collapsed', entity_view, required=False).lower() == 'true'
+
+            sorted_sources = sorted(self._graph.objects(entity_view, WIRE_M['hasSource']), key=lambda source: possible_int(self._get_field(WIRE, 'index', source, required=False)))
+            sorted_targets = sorted(self._graph.objects(entity_view, WIRE_M['hasTarget']), key=lambda target: possible_int(self._get_field(WIRE, 'index', target, required=False)))
+
+            component_view_description['endpoints'] = {}
+            component_view_description['endpoints']['source'] = [self._get_field(RDFS, 'label', sourc) for sourc in sorted_sources]
+            component_view_description['endpoints']['target'] = [self._get_field(RDFS, 'label', targ) for targ in sorted_targets]
+
+            position = self._parse_position(entity_view)
+            if position is not None:
+                component_view_description['position'] = position
+
+    def _parse_position(self, node, relation_name='hasPosition', default=None):
+            position_node = self._get_field(WIRE_M, relation_name, node, id_=True, default=None, required=False)
+            if position_node is not None:
+                return {
+                    'x': int(self._get_field(WIRE_M, 'x', position_node)),
+                    'y': int(self._get_field(WIRE_M, 'y', position_node))
+                }
+
+            return default
+
+    def _join_endpoint_name(self, endpointView):
+        endpoint = {
+            'id': self._get_field(WIRE_M, 'id', endpointView),
+            'endpoint': self._get_field(WIRE_M, 'endpoint', endpointView),
+            'type': self._get_field(WIRE, 'type', endpointView),
+        }
+
+        return "%s/%s/%s" % (endpoint['type'], endpoint['id'], endpoint['endpoint'])
+
+    def _parse_wiring_connections(self, element, behaviour):
+
+        behaviour['connections'] = []
+
+        for connection in self._graph.objects(element, WIRE_M['hasConnectionView']):
+            connection_info = {}
+
+            for source in self._graph.objects(connection, WIRE_M['hasSourceEndpoint']):
+                connection_info['sourcename'] = self._join_endpoint_name(source)
+                break
+            else:
+                raise TemplateParseException(_('missing required field: hasSourceEndpoint'))
+
+            connection_info['sourcehandle'] = self._parse_position(connection, relation_name='hasSourceHandlePosition', default="auto")
+
+            for target in self._graph.objects(connection, WIRE_M['hasTargetEndpoint']):
+                connection_info['targetname'] = self._join_endpoint_name(target)
+                break
+            else:
+                raise TemplateParseException(_('missing required field: hasTargetEndpoint'))
+
+            connection_info['targethandle'] = self._parse_position(connection, relation_name='hasTargetHandlePosition', default="auto")
+
+            behaviour['connections'].append(connection_info)
+
+    def _parse_wiring_behaviours(self, wiring_element):
+
+        visualdescription = {
+            'behaviours': [],
+        }
+
+        self._parse_wiring_components(wiring_element, visualdescription)
+        self._parse_wiring_connections(wiring_element, visualdescription)
+
+        sorted_behaviours = sorted(self._graph.objects(wiring_element, WIRE_M['hasBehaviour']), key=lambda behaviour: possible_int(self._get_field(WIRE, 'index', behaviour, required=False)))
+        for view in sorted_behaviours:
+            behaviour = {
+                'title': self._get_field(RDFS, 'label', view),
+                'description': self._get_field(DCTERMS, 'description', view),
+            }
+
+            self._parse_wiring_components(view, behaviour)
+            self._parse_wiring_connections(view, behaviour)
+
+            visualdescription['behaviours'].append(behaviour)
+
+        self._info['wiring']['visualdescription'] = visualdescription
+
     def _parse_wiring_views(self, wiring_element):
 
         wiring_views = []
 
         for view in self._graph.objects(wiring_element, WIRE_M['hasWiringView']):
-            element_view = {}
-            element_view['label'] = self._get_field(RDFS, 'label', view)
-            element_view['iwidgets'] = {}
-            element_view['operators'] = {}
+            element_view = {
+                'label': self._get_field(RDFS, 'label', view),
+                'iwidgets': {},
+                'operators': {},
+                'connections': []
+            }
 
             for entity_view in self._graph.objects(view, WIRE_M['hasView']):
 
@@ -387,7 +534,15 @@ class RDFTemplateParser(object):
             wiring_views.append(element_view)
         self._info['wiring']['views'] = wiring_views
 
-    def _parse_widget_info(self):
+        # TODO: update to the new wiring format
+        inputs = self._info['wiring']['inputs']
+        outputs = self._info['wiring']['outputs']
+        self._info['wiring'] = parse_wiring_old_version(self._info['wiring'])
+        self._info['wiring']['inputs'] = inputs
+        self._info['wiring']['outputs'] = outputs
+        # END TODO
+
+    def _parse_component_info(self):
 
         # Preference info
         self._info['preferences'] = []
@@ -406,6 +561,8 @@ class RDFTemplateParser(object):
                 'default': self._get_field(WIRE, 'default', preference, required=False),
                 'value': self._get_field(WIRE, 'value', preference, required=False, default=None),
                 'secure': self._get_field(WIRE, 'secure', preference, required=False).lower() == 'true',
+                'multiuser': False,
+                'required': self._get_field(WIRE, 'required', preference, required=False).lower() == 'true',
             }
             if preference_info['type'] == 'list':
                 preference_info['options'] = []
@@ -432,29 +589,47 @@ class RDFTemplateParser(object):
                 'description': self._get_translation_field(DCTERMS, 'description', prop, var_name + '_description', required=False, type='vdef', variable=var_name, field='description'),
                 'default': self._get_field(WIRE, 'default', prop, required=False),
                 'secure': self._get_field(WIRE, 'secure', prop, required=False).lower() == 'true',
+                'multiuser': self._get_field(WIRE, 'multiuser', prop, required=False).lower() == 'true',
             })
 
         self._parse_wiring_info()
 
         if self._info['type'] == 'widget':
             # It contains the widget code
-            xhtml_element = self._get_field(USDL, 'utilizedResource', self._rootURI, id_=True)
+            self._info['altcontents'] = []
+            sorted_contents = sorted(self._graph.objects(self._rootURI, USDL['utilizedResource']), key=lambda contents: possible_int(self._get_field(WIRE, 'index', contents, required=False)))
 
-            self._info['code_url'] = unicode(xhtml_element)
-            content_type, parameters = parse_mime_type(self._get_field(DCTERMS, 'format', xhtml_element, required=False))
+            for contents_node in sorted_contents:
+                contents_info = {
+                    'src': text_type(contents_node),
+                }
+                contents_info['scope'] = self._get_field(WIRE, 'contentsScope', contents_node, required=False)
+                contents_info['contenttype'] = 'text/html'
+                contents_info['charset'] = 'utf-8'
 
-            self._info['code_content_type'] = 'text/html'
-            self._info['code_charset'] = 'utf-8'
-            if content_type != '':
-                self._info['code_content_type'] = content_type
-                if 'charset' in parameters:
-                    self._info['code_charset'] = parameters['charset'].lower()
+                contents_format = self._get_field(DCTERMS, 'format', contents_node, required=False)
 
-            elif len(parameters) > 1:
-                raise Exception('Invalid code content type')
+                if contents_format.strip() != '':
+                    try:
+                        contenttype, parameters = parse_mime_type(contents_format)
+                    except:
+                        raise TemplateParseException('Invalid code content type: %s' % contents_format)
 
-            self._info['code_uses_platform_style'] = self._get_field(WIRE, 'usePlatformStyle', xhtml_element, required=False).lower() == 'true'
-            self._info['code_cacheable'] = self._get_field(WIRE, 'codeCacheable', xhtml_element, required=False, default='true').lower() == 'true'
+                    contents_info['contenttype'] = contenttype
+                    if 'charset' in parameters:
+                        contents_info['charset'] = parameters['charset'].lower()
+                        del parameters['charset']
+
+                    if len(parameters) > 0:
+                        raise TemplateParseException('Invalid code content type: %s' % contents_format)
+
+                if contents_info['scope'] == '':
+                    del contents_info['scope']
+                    contents_info['useplatformstyle'] = self._get_field(WIRE, 'usePlatformStyle', contents_node, required=False).lower() == 'true'
+                    contents_info['cacheable'] = self._get_field(WIRE, 'codeCacheable', contents_node, required=False, default='true').lower() == 'true'
+                    self._info['contents'] = contents_info
+                else:
+                    self._info['altcontents'].append(contents_info)
 
             rendering_element = self._get_field(WIRE, 'hasPlatformRendering', self._rootURI, id_=True, required=False)
 
@@ -469,7 +644,7 @@ class RDFTemplateParser(object):
 
             self._info['js_files'] = []
             for js_element in sorted_js_files:
-                self._info['js_files'].append(unicode(js_element))
+                self._info['js_files'].append(text_type(js_element))
 
             if not len(self._info['js_files']) > 0:
                 raise TemplateParseException(_('Missing required field: Javascript files'))
@@ -491,10 +666,27 @@ class RDFTemplateParser(object):
         ordered_params = sorted(self._graph.objects(self._rootURI, WIRE_M['hasMashupParam']), key=lambda raw_param: possible_int(self._get_field(WIRE, 'index', raw_param, required=False)))
         self._info['params'] = []
         for param in ordered_params:
+            var_name = self._get_field(DCTERMS, 'title', param, required=True)
             self._info['params'].append({
-                'name': self._get_field(DCTERMS, 'title', param),
-                'label': self._get_field(RDFS, 'label', param),
+                'name': var_name,
+                'label': self._get_translation_field(RDFS, 'label', param, var_name + '_label', required=True, type='vdef', variable=var_name, field='label'),
                 'type': self._get_field(WIRE, 'type', param),
+                'description': self._get_translation_field(DCTERMS, 'description', param, var_name + '_description', required=False, type='vdef', variable=var_name, field='description'),
+                'readonly': self._get_field(WIRE, 'readonly', param, required=False).lower() == 'true',
+                'default': self._get_field(WIRE, 'default', param, required=False),
+                'value': self._get_field(WIRE, 'value', param, required=False, default=None),
+                'required': self._get_field(WIRE, 'required', param, required=False, default="true").lower() == 'true',
+            })
+
+        self._info['embedded'] = []
+        for component in self._graph.objects(self._rootURI, WIRE_M['hasEmbeddedResource']):
+            vendor = self._get_field(USDL, 'hasProvider', component, id_=True, required=True)
+
+            self._info['embedded'].append({
+                'vendor': self._get_field(FOAF, 'name', vendor),
+                'name': self._get_field(RDFS, 'label', component),
+                'version': self._get_field(USDL, 'versionInfo', component),
+                'src': text_type(component)
             })
 
         ordered_tabs = sorted(self._graph.objects(self._rootURI, WIRE_M['hasTab']), key=lambda raw_tab: possible_int(self._get_field(WIRE, 'index', raw_tab, required=False)))
@@ -503,6 +695,7 @@ class RDFTemplateParser(object):
         for tab in ordered_tabs:
             tab_info = {
                 'name': self._get_field(DCTERMS, 'title', tab),
+                'title': self._get_field(WIRE, 'displayName', tab, required=False),
                 'preferences': {},
                 'resources': [],
             }
@@ -510,18 +703,18 @@ class RDFTemplateParser(object):
             for preference in self._graph.objects(tab, WIRE_M['hasTabPreference']):
                 tab_info['preferences'][self._get_field(DCTERMS, 'title', preference)] = self._get_field(WIRE, 'value', preference)
 
-            for resource in self._graph.objects(tab, WIRE_M['hasiWidget']):
-                position = self._get_field(WIRE_M, 'hasPosition', resource, id_=True, required=False)
-                rendering = self._get_field(WIRE_M, 'hasiWidgetRendering', resource, id_=True, required=False)
-                vendor = self._get_field(USDL, 'hasProvider', resource, id_=True, required=False)
+            for widget in self._graph.objects(tab, WIRE_M['hasiWidget']):
+                position = self._get_field(WIRE_M, 'hasPosition', widget, id_=True, required=False)
+                rendering = self._get_field(WIRE_M, 'hasiWidgetRendering', widget, id_=True, required=False)
+                vendor = self._get_field(USDL, 'hasProvider', widget, id_=True, required=True)
 
-                resource_info = {
-                    'id': self._get_field(WIRE_M, 'iWidgetId', resource),
-                    'name': self._get_field(RDFS, 'label', resource),
+                widget_info = {
+                    'id': self._get_field(WIRE_M, 'iWidgetId', widget),
                     'vendor': self._get_field(FOAF, 'name', vendor),
-                    'version': self._get_field(USDL, 'versionInfo', resource),
-                    'title': self._get_field(DCTERMS, 'title', resource),
-                    'readonly': self._get_field(WIRE_M, 'readonly', resource, required=False).lower() == 'true',
+                    'name': self._get_field(RDFS, 'label', widget),
+                    'version': self._get_field(USDL, 'versionInfo', widget),
+                    'title': self._get_field(DCTERMS, 'title', widget),
+                    'readonly': self._get_field(WIRE_M, 'readonly', widget, required=False).lower() == 'true',
                     'properties': {},
                     'preferences': {},
                     'position': {
@@ -538,44 +731,25 @@ class RDFTemplateParser(object):
                     },
                 }
 
-                for prop in self._graph.objects(resource, WIRE_M['hasiWidgetProperty']):
-                    resource_info['properties'][self._get_field(DCTERMS, 'title', prop)] = {
+                for prop in self._graph.objects(widget, WIRE_M['hasiWidgetProperty']):
+                    widget_info['properties'][self._get_field(DCTERMS, 'title', prop)] = {
                         'readonly': self._get_field(WIRE_M, 'readonly', prop, required=False).lower() == 'true',
-                        'value': self._get_field(WIRE, 'value', prop, required=False),
+                        'value': self._get_field(WIRE, 'value', prop, default=None, required=False),
                     }
 
-                for pref in self._graph.objects(resource, WIRE_M['hasiWidgetPreference']):
-                    resource_info['preferences'][self._get_field(DCTERMS, 'title', pref)] = {
+                for pref in self._graph.objects(widget, WIRE_M['hasiWidgetPreference']):
+                    widget_info['preferences'][self._get_field(DCTERMS, 'title', pref)] = {
                         'readonly': self._get_field(WIRE_M, 'readonly', pref, required=False).lower() == 'true',
                         'hidden': self._get_field(WIRE_M, 'hidden', pref, required=False).lower() == 'true',
-                        'value': self._get_field(WIRE, 'value', pref, required=False),
+                        'value': self._get_field(WIRE, 'value', pref, default=None, required=False),
                     }
 
-                tab_info['resources'].append(resource_info)
+                tab_info['resources'].append(widget_info)
 
             tabs.append(tab_info)
 
         self._info['tabs'] = tabs
-
-        self._parse_wiring_info(wiring_property='hasMashupWiring', parse_connections=True)
-        #wiring_element = self._xpath(WIRING_XPATH, self._doc)[0]
-
-    def typeText2typeCode(self, typeText):
-        mapping = {
-            'text': 'S',
-            'number': 'N',
-            'date': 'D',
-            'boolean': 'B',
-            'list': 'L',
-            'password': 'P',
-        }
-        if typeText in mapping:
-            return mapping[typeText]
-        else:
-            raise TemplateParseException(_("ERROR: unkown TEXT TYPE ") + typeText)
-
-    def get_contents(self):
-        return self._graph.serialize(format='pretty-xml')
+        self._parse_wiring_info(wiring_property='hasMashupWiring')
 
     def get_resource_type(self):
         return self._info['type']
@@ -589,9 +763,6 @@ class RDFTemplateParser(object):
     def get_resource_version(self):
         return self._info['version']
 
-    def get_resource_basic_info(self):
-        return self._info
-
     def get_resource_info(self):
         if not self._parsed:
             self._parse_extra_info()
