@@ -18,7 +18,6 @@
 # along with Wirecloud.  If not, see <http://www.gnu.org/licenses/>.
 
 from cStringIO import StringIO
-import json
 import zipfile
 
 from django.contrib.auth.models import User
@@ -26,10 +25,12 @@ from django.http import HttpResponse, Http404
 from django.shortcuts import get_object_or_404
 from django.utils.translation import ugettext as _
 from django.views.decorators.http import require_GET, require_POST
+import six
+from six import string_types
 
 from wirecloud.catalogue.models import CatalogueResource
 from wirecloud.commons.utils.downloader import download_http_content
-from wirecloud.commons.utils.http import build_error_response, get_content_type
+from wirecloud.commons.utils.http import build_error_response, get_content_type, parse_json_request
 from wirecloud.commons.utils.template import TemplateParser, TemplateParseException
 from wirecloud.commons.utils.template.writers.rdf import write_rdf_description
 from wirecloud.commons.utils.transaction import commit_on_http_success
@@ -56,18 +57,14 @@ def parse_username(tenant_id):
 @commit_on_http_success
 def add_tenant(request):
 
-    try:
-        data = json.loads(request.body)
-    except ValueError as e:
-        msg = _("malformed json data: %s") % unicode(e)
-        return build_error_response(request, 400, msg)
+    data = parse_json_request(request)
 
     id_4CaaSt = data['4CaaStID']
 
     if id_4CaaSt is None:
         return build_error_response(request, 400, _('Missing 4CaaStID'))
 
-    if not isinstance(id_4CaaSt, basestring) or id_4CaaSt.strip() == '':
+    if not isinstance(id_4CaaSt, string_types) or id_4CaaSt.strip() == '':
         return build_error_response(request, 400, _('Invalid 4CaaStID'))
 
     username = parse_username(id_4CaaSt)
@@ -96,18 +93,14 @@ def add_tenant(request):
 @commit_on_http_success
 def remove_tenant(request):
 
-    try:
-        data = json.loads(request.body)
-    except ValueError as e:
-        msg = _("malformed json data: %s") % unicode(e)
-        return build_error_response(request, 400, msg)
+    data = parse_json_request(request)
 
     id_4CaaSt = data.get('4CaaStID')
 
     if id_4CaaSt is None:
         return build_error_response(request, 400, _('Missing 4CaaStID'))
 
-    if not isinstance(id_4CaaSt, basestring) or id_4CaaSt.strip() == '':
+    if not isinstance(id_4CaaSt, string_types) or id_4CaaSt.strip() == '':
         return build_error_response(request, 400, _('Invalid 4CaaStID'))
 
     username = parse_username(id_4CaaSt)
@@ -123,17 +116,14 @@ def remove_tenant(request):
 
     return HttpResponse(status=204)
 
+
 def _parse_ac_request(request):
 
     fileURL = None
     file_contents = None
     content_type = get_content_type(request)[0]
 
-    try:
-        data = json.loads(request.body)
-    except Exception as e:
-        msg = _("malformed json data: %s") % unicode(e)
-        return build_error_response(request, 400, msg)
+    data = parse_json_request(request)
 
     if 'url' not in data:
         return build_error_response(request, 400, _('Missing widget URL'))
@@ -144,7 +134,7 @@ def _parse_ac_request(request):
     if id_4CaaSt is None:
         return build_error_response(request, 400, _('Missing 4CaaStID'))
 
-    if not isinstance(id_4CaaSt, basestring) or id_4CaaSt.strip() == '':
+    if not isinstance(id_4CaaSt, string_types) or id_4CaaSt.strip() == '':
         return build_error_response(request, 400, _('Invalid 4CaaStID'))
 
     try:
@@ -160,7 +150,7 @@ def _parse_ac_request(request):
     template_info = template.get_resource_info()
     template_info['name'] += '@' + id_4CaaSt
 
-    for pref_name, pref_value in data.get('preferences', {}).iteritems():
+    for pref_name, pref_value in six.iteritems(data.get('preferences', {})):
         for widget_pref_index, widget_pref in enumerate(template_info['preferences']):
             if widget_pref['name'] == pref_name:
                 template_info['preferences'][widget_pref_index]['readonly'] = True
@@ -183,6 +173,7 @@ def _parse_ac_request(request):
 
     return id_4CaaSt, file_contents, fileURL
 
+
 @require_POST
 @commit_on_http_success
 def deploy_tenant_ac(request):
@@ -201,18 +192,19 @@ def deploy_tenant_ac(request):
     # Install uploaded MAC resource
     try:
 
-        resource = install_resource_to_user(user, file_contents=wgt_file, templateURL=fileURL, packaged=True)
+        resource = install_resource_to_user(user, file_contents=wgt_file)
 
     except TemplateParseException as e:
 
-        return build_error_response(request, 400, unicode(e.msg))
+        return build_error_response(request, 400, e.msg)
 
     # Create a workspace if the resource is a mashup
-    if resource.resource_type() == 'mashup' and not Workspace.objects.filter(creator=user, name=resource.display_name).exists():
+    if resource.resource_type() == 'mashup' and not Workspace.objects.filter(creator=user, name=resource.short_name).exists():
         buildWorkspaceFromTemplate(resource.get_template(), user, True)
 
     return HttpResponse(status=204)
 
+
 @require_POST
 @commit_on_http_success
 def undeploy_tenant_ac(request):
@@ -263,7 +255,7 @@ def add_saas_tenant(request, creator, workspace):
     if id_4CaaSt is None:
         return build_error_response(request, 400, _('Missing 4CaaStID'))
 
-    if not isinstance(id_4CaaSt, basestring) or id_4CaaSt.strip() == '':
+    if not isinstance(id_4CaaSt, string_types) or id_4CaaSt.strip() == '':
         return build_error_response(request, 400, _('Invalid 4CaaStID'))
 
     username = parse_username(id_4CaaSt)
@@ -298,7 +290,7 @@ def remove_saas_tenant(request, creator, workspace):
     if id_4CaaSt is None:
         return build_error_response(request, 400, _('Missing 4CaaStID'))
 
-    if not isinstance(id_4CaaSt, basestring) or id_4CaaSt.strip() == '':
+    if not isinstance(id_4CaaSt, string_types) or id_4CaaSt.strip() == '':
         return build_error_response(request, 400, _('Invalid 4CaaStID'))
 
     username = parse_username(id_4CaaSt)
