@@ -21,7 +21,7 @@ import json
 from six.moves.urllib.parse import parse_qs, urlparse
 
 from django.core.urlresolvers import reverse
-from django.test import Client
+from django.test import Client, TransactionTestCase
 from django.test.utils import override_settings
 from django.utils.http import urlencode
 from mock import patch
@@ -31,16 +31,18 @@ from wirecloud.commons.utils.testcases import WirecloudTestCase
 from wirecloud.platform.plugins import clear_cache
 
 
-@override_settings(INSTALLED_APPS=BASE_APPS + ('wirecloud.catalogue', 'wirecloud.platform', 'wirecloud.oauth2provider'))
-class Oauth2TestCase(WirecloudTestCase):
+@override_settings(INSTALLED_APPS=BASE_APPS + ('wirecloud.catalogue', 'wirecloud.platform', 'wirecloud.oauth2provider', 'haystack'))
+class Oauth2TestCase(WirecloudTestCase, TransactionTestCase):
 
     fixtures = ('selenium_test_data', 'oauth2_test_data')
     tags = ('wirecloud-oauth2provider', 'wirecloud-noselenium')
+    populate = False
+    use_search_indexes = False
 
     @classmethod
     def setUpClass(cls):
 
-        WirecloudTestCase.setUpClass.__func__(cls)
+        super(Oauth2TestCase, cls).setUpClass()
 
         cls.client = Client()
         cls.user_client = Client()
@@ -50,12 +52,14 @@ class Oauth2TestCase(WirecloudTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        WirecloudTestCase.tearDownClass.__func__(cls)
+        super(Oauth2TestCase, cls).tearDownClass()
 
         # Clear WireCloud plugins cache
         clear_cache()
 
     def setUp(self):
+        super(Oauth2TestCase, self).setUp()
+
         self.user_client.login(username='normuser', password='admin')
 
     def _check_token(self, token, endpoint='wirecloud.resource_collection'):
