@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2012-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2012-2015 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -24,40 +24,40 @@ from relatives.utils import object_edit_link
 from wirecloud.platform import models
 
 
-admin.site.register(models.Constant)
+def get_workspace(iwidget):
+    return iwidget.tab.workspace
+get_workspace.short_description = 'Workspace'
 
-admin.site.register(models.Position)
-admin.site.register(models.IWidget)
-admin.site.register(models.Variable)
+
+class IWidgetAdmin(admin.ModelAdmin):
+
+    list_display = ('id', get_workspace, 'tab', 'widget', 'name')
+
+
+class TeamInline(admin.StackedInline):
+
+    model = models.Team
+    fields = ('name', 'users')
+    ordering = ('name',)
+    extra = 0
+
+
+class OrganizationAdmin(admin.ModelAdmin):
+
+    list_display = ('user',)
+    list_display_links = ('user',)
+    ordering = ('user',)
+    inlines = (TeamInline,)
 
 
 class MarketAdmin(admin.ModelAdmin):
     list_display = ('user', 'name')
     ordering = ('user', 'name')
-admin.site.register(models.Market, MarketAdmin)
+
 
 class MarketUserDataAdmin(admin.ModelAdmin):
     list_display = ('market', 'user', 'name', 'value')
     ordering = ('market', 'user', 'name')
-admin.site.register(models.MarketUserData, MarketUserDataAdmin)
-
-class VariableDefInline(admin.StackedInline):
-
-    model = models.VariableDef
-    extra = 0
-
-class WidgetAdmin(admin.ModelAdmin):
-
-    inlines = (VariableDefInline,)
-
-
-admin.site.register(models.Widget, WidgetAdmin)
-admin.site.register(models.XHTML)
-
-admin.site.register(models.UserPrefOption)
-admin.site.register(models.VariableDefAttr)
-
-admin.site.register(models.PlatformPreference)
 
 
 class WorkspacePreferenceInline(admin.TabularInline):
@@ -91,10 +91,21 @@ class TabAdmin(admin.ModelAdmin):
 
 
 class WorkspaceAdmin(admin.ModelAdmin):
-    list_display = ('creator', 'name')
+
+    search_fields = ('creator__username', 'name', 'description')
+    list_display = ('creator', 'name', 'description')
     ordering = ('creator', 'name')
     inlines = (WorkspacePreferenceInline, TabInline,)
 
+
+admin.site.register(models.Constant)
+admin.site.register(models.IWidget, IWidgetAdmin)
+admin.site.register(models.Organization, OrganizationAdmin)
+admin.site.register(models.Market, MarketAdmin)
+admin.site.register(models.MarketUserData, MarketUserDataAdmin)
+admin.site.register(models.Widget)
+admin.site.register(models.XHTML)
+admin.site.register(models.PlatformPreference)
 admin.site.register(models.Workspace, WorkspaceAdmin)
 admin.site.register(models.Tab, TabAdmin)
 admin.site.register(models.UserWorkspace)
