@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2012-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2012-2017 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -17,12 +17,15 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with Wirecloud.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import unicode_literals
+
 import json
 import os
 import requests
-from urlparse import urljoin
+from six.moves.urllib.parse import urljoin
 
 from django.utils.encoding import iri_to_uri
+from django.utils.translation import ugettext as _
 
 from wirecloud.catalogue.models import CatalogueResource
 import wirecloud.catalogue.utils as catalogue_utils
@@ -44,44 +47,6 @@ class WirecloudCatalogueManager(MarketManager):
         self._name = name
         self._options = options
 
-    def search_resource(self, vendor, name, version, user):
-
-        if self._name == 'local':
-            resources = CatalogueResource.objects.filter(vendor=vendor, short_name=name, version=version)[:1]
-
-            if len(resources) == 1:
-                resource = resources[0]
-
-                if resource.template_uri.startswith(('http://', 'https://')):
-                    downloaded_file = download_http_content(resource.template_uri, user=user)
-                else:
-                    base_dir = catalogue_utils.wgt_deployer.get_base_dir(vendor, name, version)
-                    downloaded_file = download_local_file(os.path.join(base_dir, resource.template_uri))
-
-                return {
-                    'downloaded_file': downloaded_file,
-                    'template_url': resource.template_uri,
-                    'packaged': resource.fromWGT
-                }
-            else:
-                return None
-        else:
-
-            path = '/'.join(('catalogue', 'resource', vendor, name, version))
-            url = iri_to_uri(urljoin(self._options['url'], path))
-            response = requests.get(url)
-
-            if response.status_code == 200:
-                data = json.loads(response.content)
-                downloaded_file = download_http_content(data['uriTemplate'], user=user)
-                return {
-                    'downloaded_file': downloaded_file,
-                    'template_url': data['uriTemplate'],
-                    'packaged': data['packaged']
-                }
-            else:
-                return None
-
     def publish(self, endpoint, wgt_file, user, request=None, template=None):
 
         if self._name == 'local':
@@ -89,6 +54,10 @@ class WirecloudCatalogueManager(MarketManager):
             if template is None:
                 template = TemplateParser(wgt_file.get_template())
 
-            install_resource_to_user(user, file_contents=wgt_file, packaged=True, raise_conflicts=True)
+            added, resource = install_resource_to_user(user, file_contents=wgt_file, packaged=True, raise_conflicts=True)
+            if not added:
+                raise Exception(_('Resource already exists %(resource_id)s') % {'resource_id': resource.local_uri_part})
+
+            return resource
         else:
             raise Exception('TODO')
