@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2012-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2012-2017 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -26,11 +26,12 @@ from django.db.models import Q
 from django.http import HttpResponse
 from django.shortcuts import get_object_or_404
 from django.utils.translation import ugettext as _
+from six import text_type
 
 from wirecloud.catalogue.models import CatalogueResource
 from wirecloud.catalogue import utils as catalogue
 from wirecloud.commons.baseviews import Resource, Service
-from wirecloud.commons.utils.http import authentication_required, build_error_response, supported_request_mime_types
+from wirecloud.commons.utils.http import authentication_required, build_error_response, consumes, parse_json_request, validate_url_param
 from wirecloud.commons.utils.transaction import commit_on_http_success
 from wirecloud.commons.utils.wgt import WgtFile
 from wirecloud.platform.markets.utils import get_market_managers
@@ -41,63 +42,61 @@ class MarketCollection(Resource):
 
     @authentication_required
     def read(self, request):
-        result = {}
-
-        for market in Market.objects.filter(Q(user=None) | Q(user=request.user)):
-            market_key = unicode(market)
-            market_data = json.loads(market.options)
+        result = []
 
+        for market in Market.objects.filter(Q(public=True) | Q(user=request.user)).order_by('user'):
+            market_data = market.options
             market_data['name'] = market.name
-
-            if market.user is not None:
-                market_data['user'] = market.user.username
-            else:
-                market_data['user'] = None
+            market_data['user'] = market.user.username
+            market_data['public'] = market.public
 
             market_data['permissions'] = {
                 'delete': request.user.is_superuser or market.user == request.user
             }
 
-            result[market_key] = market_data
+            result.append(market_data)
 
-        return HttpResponse(json.dumps(result), content_type='application/json; charset=UTF-8')
+        return HttpResponse(json.dumps(result, sort_keys=True), content_type='application/json; charset=UTF-8')
 
     @authentication_required
-    @supported_request_mime_types(('application/json'))
+    @consumes(('application/json'))
     @commit_on_http_success
     def create(self, request):
 
-        try:
-            received_data = json.loads(request.body)
-        except ValueError as e:
-            msg = _("malformed json data: %s") % unicode(e)
-            return build_error_response(request, 400, msg)
+        received_data = parse_json_request(request)
+        received_data.setdefault("public", False)
 
-        if 'user' not in received_data['options'] or received_data['options']['user'] == request.user.username:
-            user_entry = request.user
-        elif received_data['options'].get('user', None) is not None:
-            user_entry = User.objects.get(username=received_data['options']['user'])
+        validate_url_param(request, 'url', received_data['url'])
+
+        if 'user' not in received_data or received_data['user'] == request.user.username:
+            target_user = request.user
         else:
-            user_entry = None
+            try:
+                target_user = User.objects.get(username=received_data['user'])
+            except:
+                return build_error_response(request, 422, _("invalid user option"))
 
-        if (user_entry is None or user_entry != request.user) and not request.user.is_superuser:
-            return build_error_response(request, 403, _("You don't have permissions for adding public marketplaces"))
+        if target_user != request.user and not request.user.is_superuser:
+            return build_error_response(request, 403, _("You don't have permissions for adding marketplaces in name of other user"))
 
-        if 'user' in received_data['options']:
-            del received_data['options']['user']
+        received_data['user'] = target_user.username
 
         try:
-            Market.objects.create(user=user_entry, name=received_data['name'], options=json.dumps(received_data['options']))
+            Market.objects.create(user=target_user, name=received_data['name'], public=received_data['public'], options=received_data)
         except IntegrityError:
             return build_error_response(request, 409, 'Market name already in use')
 
+        market_managers = get_market_managers(target_user)
+        market_managers[target_user.username + '/' + received_data['name']].create(target_user)
+
         return HttpResponse(status=201)
 
 
 class MarketEntry(Resource):
 
     @authentication_required
-    def delete(self, request, market, user=None):
+    @commit_on_http_success
+    def delete(self, request, user, market):
 
         if user is None and (not request.user.is_superuser or market == 'local'):
             return build_error_response(request, 403, _('You are not allowed to delete this market'))
@@ -105,32 +104,32 @@ class MarketEntry(Resource):
         if user != request.user.username and not request.user.is_superuser:
             return build_error_response(request, 403, _('You are not allowed to delete this market'))
 
+        if user == request.user.username:
+            target_user = request.user
+        else:
+            target_user = get_object_or_404(User, username=user)
+
+        market_managers = get_market_managers(target_user)
+
         get_object_or_404(Market, user__username=user, name=market).delete()
+        market_managers[user + '/' + market].delete()
 
         return HttpResponse(status=204)
 
-    def update(self, request, market):
-        pass
-
 
 class PublishService(Service):
 
     @authentication_required
-    @supported_request_mime_types(('application/json'))
+    @consumes(('application/json'))
     def process(self, request):
 
-        try:
-            data = json.loads(request.body)
-        except ValueError as e:
-            msg = _("malformed json data: %s") % unicode(e)
-            return build_error_response(request, 400, msg)
+        data = parse_json_request(request)
 
         (resource_vendor, resource_name, resource_version) = data['resource'].split('/')
         resource = get_object_or_404(CatalogueResource, vendor=resource_vendor, short_name=resource_name, version=resource_version)
 
-        if not resource.fromWGT:
-            msg = _('Only packaged resources can be published')
-            return build_error_response(request, 400, msg)
+        if not resource.is_available_for(request.user):
+            return build_error_response(request, 403, _('You are not allowed to delete this market'))
 
         base_dir = catalogue.wgt_deployer.get_base_dir(resource_vendor, resource_name, resource_version)
         wgt_file = WgtFile(os.path.join(base_dir, resource.template_uri))
@@ -142,11 +141,11 @@ class PublishService(Service):
             try:
                 market_managers[market_endpoint['market']].publish(market_endpoint, wgt_file, request.user, request=request)
             except Exception as e:
-                errors[market_endpoint['market']] = unicode(e)
+                errors[market_endpoint['market']] = text_type(e)
 
         if len(errors) == 0:
             return HttpResponse(status=204)
         elif len(errors) == len(data['marketplaces']):
-            return HttpResponse(json.dumps(errors), status=502, content_type='application/json; charset=UTF-8')
+            return build_error_response(request, 502, _('Something went wrong (see details for more info)'), details=errors)
         else:
-            return HttpResponse(json.dumps(errors), status=200, content_type='application/json; charset=UTF-8')
+            return build_error_response(request, 200, _('Something went wrong (see details for more info)'), details=errors)
