@@ -1,5 +1,5 @@
 /*
- *     Copyright (c) 2012-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+ *     Copyright (c) 2012-2017 CoNWeT Lab., Universidad Politécnica de Madrid
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -19,119 +19,120 @@
  *
  */
 
-/*global gettext, StyledElements, Wirecloud*/
+/* globals StyledElements, Wirecloud */
 
-(function () {
+
+(function (utils) {
 
     "use strict";
 
-    var initEmptyCatalogueInfoBox = function initEmptyCatalogueInfoBox() {
-        // Tutorial layer for empty catalogues
-        this.emptyBox = document.createElement('div');
-        this.emptyBox.className = 'catalogueEmptyBox';
+    var builder = new StyledElements.GUIBuilder();
 
-        var wrapper = document.createElement('div');
-        wrapper.className = 'alert alert-info';
+    var initEmptyCatalogueInfoBox = function initEmptyCatalogueInfoBox(title, message) {
 
-        // Title
-        var pTitle = document.createElement('h4');
-        pTitle.textContent = gettext("Empty Marketplace!");
-        wrapper.appendChild(pTitle);
+        // Build the message box used when there are no resources in the catalogue
+        var layer = builder.DEFAULT_OPENING + '<div class="catalogueEmptyBox"><div class="alert alert-info"><h4><t:title/></h4><p><t:message/></p></div></div>' + builder.DEFAULT_CLOSING;
 
-        // Message
-        var message = document.createElement('p');
-        message.innerHTML = gettext("This is an empty Marketplace. You can upload widgets or operators using the button to the right of the name of the marketplace");
-        wrapper.appendChild(message);
-
-        this.emptyBox.appendChild(wrapper);
+        this.emptyBox = builder.parse(layer, {title: title, message: message}).elements[0];
     };
 
     var CatalogueSearchView = function CatalogueSearchView(id, options) {
-        var builder, context, extra_context, resource_template;
+        var context, extra_context, resource_template;
+
+        options = utils.merge({
+            // Default options
+            emptyTitle: utils.gettext("Empty Marketplace!"),
+            emptyMessage: utils.gettext("This marketplace is empty, that is, it does not provide any resource at this time.")
+        }, options);
 
         options['class'] = 'search_interface loading';
         this.catalogue = options.catalogue;
         StyledElements.Alternative.call(this, id, options);
 
         if (options.gui_template == null) {
-            options.gui_template = 'wirecloud_catalogue_search_interface';
+            options.gui_template = 'wirecloud/catalogue/search_interface';
         }
-
-        builder = new StyledElements.GUIBuilder();
         this.source = new StyledElements.PaginatedSource({
             'pageSize': 30,
-            'order_by': '-popularity',
+            'order_by': '-creation_date',
             'keywords': '',
             'scope': 'all',
             'requestFunc': this._search.bind(this),
-            'processFunc': function (elements) {
-                var i, resource;
+            'processFunc': function (elements, search_info) {
+                var i, msg;
 
                 this.resource_list.clear();
 
-                for (i = 0; i < elements.resources.length; i += 1) {
-                    resource = elements.resources[i];
-                    this.resource_list.appendChild(this.resource_painter.paint(resource));
+                if ('corrected_query' in search_info) {
+                    msg = utils.gettext("<p>Showing results for <b><t:corrected_query/></b></p>");
+
+                    this.resource_list.appendChild(this.resource_painter.paintInfo(msg, {
+                        corrected_query: search_info.corrected_query
+                    }));
+                }
+
+                for (i = 0; i < elements.length; i += 1) {
+                    this.resource_list.appendChild(this.resource_painter.paint(elements[i]));
                 }
             }.bind(this)
         });
+        this.source.addEventListener('optionsChanged', function (source, options) {
+            if (this.scopeSelect) {
+                this.scopeSelect.setValue(options.scope);
+            }
+            if (this.simple_search_input) {
+                this.simple_search_input.setValue(options.keywords);
+            }
+            update_resetbutton.call(this, options);
+        }.bind(this));
         this.source.addEventListener('requestStart', this.disable.bind(this));
         this.source.addEventListener('requestEnd', function (pagination, error) {
             var msg;
 
             if (error != null) {
-                this.resource_painter.setError(gettext('Connection error: No resources retrieved.'));
+                this.resource_painter.setError(utils.gettext('Connection error: No resource retrieved.'));
             }
 
-            if (pagination.pCachedTotalCount === 0 && pagination.pOptions.keywords.trim() === "" && pagination.pOptions.scope === 'all') {
+            if (pagination.totalCount === 0 && pagination.options.keywords.trim() === "" && pagination.options.scope === 'all') {
                 this.resource_list.appendChild(this.emptyBox);
-            } else if (pagination.pCachedTotalCount === 0) {
-                msg = gettext("<p>We couldn't find anything for your search - <b>%(keywords)s.</b></p>" +
-                    "<p>Suggestions:</p>" +
-                    "<ul>" +
-                    "<li>Make sure all words are spelled correctly.</li>" +
-                    "<li>Try different keywords.</li>" +
-                    "<li>Try more general keywords.</li>" +
-                    "</ul>");
-                msg = interpolate(msg, {keywords: Wirecloud.Utils.escapeHTML(pagination.pOptions.keywords.trim())}, true);
+            } else if (pagination.totalCount === 0) {
+                msg = utils.gettext("<p>We couldn't find anything for your search - <b>%(keywords)s.</b></p><p>Suggestions:</p><ul><li>Make sure all words are spelled correctly.</li><li>Try different keywords.</li><li>Try more general keywords.</li></ul>");
+                msg = interpolate(msg, {keywords: utils.escapeHTML(pagination.options.keywords.trim())}, true);
                 this.resource_painter.setError(new StyledElements.Fragment(msg));
             }
 
             this.enable();
         }.bind(this));
         this.resource_list = new StyledElements.Container({'class': 'resource_list'});
-        this.simple_search_input = new StyledElements.StyledTextField();
-        this.simple_search_input.inputElement.className = 'simple_search_text';
-        this.simple_search_input.inputElement.addEventListener('keypress', this._onSearchInputKeyPress.bind(this));
-        this.simple_search_input.addEventListener('change', this._onSearchInput.bind(this));
+        this.simple_search_input = new StyledElements.TextField({'class': 'simple_search_text', 'placeholder': utils.gettext('Keywords...')});
+        this.simple_search_input.addEventListener('keydown', this._onSearchInputKeyPress.bind(this));
+        this.simple_search_input.addEventListener('change', onSearchInput.bind(this));
 
         if ('extra_context' in options) {
             extra_context = options.extra_context;
         } else {
             extra_context = {};
         }
-        context = Wirecloud.Utils.merge(extra_context, {
+        context = utils.merge(extra_context, {
             'resourcelist': this.resource_list,
             'pagination': function () {
                 return new StyledElements.PaginationInterface(this.source);
             }.bind(this),
             'reset_button': function () {
-                var button = new StyledElements.StyledButton({text: gettext('View All')});
+                var button = new StyledElements.Button({text: utils.gettext('Refresh')});
                 button.addEventListener('click', function () {
-                    this.simple_search_input.setValue('');
-                    this.source.changeOptions({'keywords': ''});
+                    this.source.changeOptions({'correct_query': true, 'keywords': '', scope: 'all'});
                 }.bind(this));
                 this.view_allbutton = button;
                 return button;
             }.bind(this),
             'orderby': function () {
-                var select = new StyledElements.StyledSelect({
-                    'initialValue': '-popularity',
+                var select = new StyledElements.Select({
+                    'initialValue': '-creation_date',
                     'initialEntries': [
-                        {'label': gettext('Popularity'), 'value': '-popularity'},
-                        {'label': gettext('Creation date'), 'value': '-creation_date'},
-                        {'label': gettext('Short name'), 'value': 'short_name'},
-                        {'label': gettext('Vendor'), 'value': 'vendor'}
+                        {'label': utils.gettext('Creation date'), 'value': '-creation_date'},
+                        {'label': utils.gettext('Title'), 'value': 'name'},
+                        {'label': utils.gettext('Vendor'), 'value': 'vendor'}
                     ]
                 });
                 select.addEventListener('change', function (select) {
@@ -140,35 +141,19 @@
                 return select;
             }.bind(this),
             'scope': function () {
-                var select = new StyledElements.StyledSelect({
+                var select = new StyledElements.Select({
                     'initialValue': 'all',
                     'initialEntries': [
-                        {'label': gettext('All'), 'value': 'all'},
-                        {'label': gettext('Widgets'), 'value': 'widget'},
-                        {'label': gettext('Mashups'), 'value': 'mashup'},
-                        {'label': gettext('Operators'), 'value': 'operator'}
+                        {'label': utils.gettext('All'), 'value': 'all'},
+                        {'label': utils.gettext('Widgets'), 'value': 'widget'},
+                        {'label': utils.gettext('Mashups'), 'value': 'mashup'},
+                        {'label': utils.gettext('Operators'), 'value': 'operator'}
                     ]
                 });
                 select.addEventListener('change', function (select) {
                     this.source.changeOptions({'scope': select.getValue()});
                 }.bind(this));
-                return select;
-            }.bind(this),
-            'widgetsperpage': function () {
-                var select = new StyledElements.StyledSelect({
-                    'initialValue': '30',
-                    'initialEntries': [
-                        {'value': '10'},
-                        {'value': '20'},
-                        {'value': '30'},
-                        {'value': '40'},
-                        {'value': '100'}
-                    ]
-                });
-                select.addEventListener('change', function (select) {
-                    this.source.changeOptions({'pageSize': select.getValue()});
-                }.bind(this));
-                this.widgetsperpage = select;
+                this.scopeSelect = select;
                 return select;
             }.bind(this),
             'searchinput': this.simple_search_input
@@ -184,15 +169,18 @@
         if ('resource_template' in options) {
             resource_template = options.resource_template;
         } else {
-            resource_template = 'catalogue_resource_template';
+            resource_template = 'wirecloud/catalogue/resource';
         }
+
         this.resource_painter = new options.resource_painter(this.catalogue,
             Wirecloud.currentTheme.templates[resource_template],
-            this.resource_list
+            this.resource_list,
+            options.resource_extra_context
         );
 
+        this.catalogue.catalogue.addEventListener('change', this.mark_outdated.bind(this));
         this.addEventListener('show', this.refresh_if_needed.bind(this));
-        initEmptyCatalogueInfoBox.call(this);
+        initEmptyCatalogueInfoBox.call(this, options.emptyTitle, options.emptyMessage);
     };
     CatalogueSearchView.prototype = new StyledElements.Alternative();
 
@@ -212,29 +200,49 @@
         }
     };
 
+    CatalogueSearchView.prototype.handleKeydownEvent = function handleKeydownEvent(key, modifiers) {
+        if ((modifiers.ctrlKey || modifiers.metaKey) && key == 'f') {
+            this.simple_search_input.focus();
+            return true;
+        }
+    };
+
     CatalogueSearchView.prototype._search = function _search(page, options, onSuccess, onError) {
         options = {
             'order_by': options.order_by,
             'search_criteria': options.keywords,
-            'search_boolean': 'AND',
             'scope': options.scope,
-            'starting_page': page,
-            'resources_per_page': options.pageSize
+            'pagenum': page,
+            'maxresults': options.pageSize
         };
         if (typeof this.catalogue.getCurrentSearchContext === 'function') {
-            options = Wirecloud.Utils.merge(options, this.catalogue.getCurrentSearchContext());
+            options = utils.merge(options, this.catalogue.getCurrentSearchContext());
         }
 
         this._last_search = Date.now();
-        this.catalogue.search(onSuccess, onError, options);
+        this.catalogue.search(options).then(search_info => {
+            onSuccess(search_info.resources, search_info);
+        }, onError);
     };
 
     CatalogueSearchView.prototype._keywordTimeoutHandler = function _keywordTimeoutHandler() {
         this.timeout = null;
-        this.source.changeOptions({'keywords': this.simple_search_input.getValue()});
+        this.source.changeOptions({'correct_query': true, 'keywords': this.simple_search_input.getValue()});
     };
 
-    CatalogueSearchView.prototype._onSearchInput = function _onSearchInput(event) {
+    var update_resetbutton = function update_resetbutton(options) {
+        var filters_applied;
+
+        filters_applied = options.keywords !== '' || options.scope !== 'all';
+
+        if (filters_applied) {
+            this.view_allbutton.setLabel(utils.gettext('Clear filters'));
+        } else {
+            this.view_allbutton.setLabel(utils.gettext('Refresh'));
+        }
+    };
+
+    var onSearchInput = function onSearchInput(event) {
 
         // Cancel current timeout
         if (this.timeout !== null) {
@@ -244,9 +252,9 @@
         this.timeout = setTimeout(this._keywordTimeoutHandler, 700);
     };
 
-    CatalogueSearchView.prototype._onSearchInputKeyPress = function _onSearchInputKeyPress(event) {
+    CatalogueSearchView.prototype._onSearchInputKeyPress = function _onSearchInputKeyPress(input, modifiers, key) {
 
-        if (event.keyCode === 13) { // enter
+        if (key === "Enter") {
 
             // Cancel current timeout
             if (this.timeout !== null) {
@@ -259,4 +267,5 @@
     };
 
     window.CatalogueSearchView = CatalogueSearchView;
-})();
+
+})(Wirecloud.Utils);
