@@ -1,5 +1,6 @@
 /*
  *     Copyright (c) 2012-2017 CoNWeT Lab., Universidad Politécnica de Madrid
+ *     Copyright (c) 2018 Future Internet Consulting and Development Solutions S.L.
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -197,10 +198,13 @@
                 onSuccess.call(this, resource);
                 onComplete.call(this);
             } else {
-                this.catalogue.getResourceDetails(resource.vendor, resource.name, {
-                    onSuccess: onSuccess.bind(this),
-                    onComplete: onComplete.bind(this)
-                });
+                this.catalogue.getResourceDetails(resource.vendor, resource.name).then(
+                    (resource) => {
+                        onSuccess.call(this, resource);
+                        onComplete.call(this);
+                    },
+                    onComplete.bind(this)
+                );
             }
         }.bind(this);
     };
@@ -224,8 +228,7 @@
         msg = utils.gettext('Do you really want to remove the "%(name)s" (vendor: "%(vendor)s", version: "%(version)s") component?');
         msg = utils.interpolate(msg, resource, true);
         return function () {
-            var dialog = new Wirecloud.ui.AlertWindowMenu();
-            dialog.setMsg(msg);
+            var dialog = new Wirecloud.ui.AlertWindowMenu(msg);
             dialog.setHandler(doRequest.bind(this));
             dialog.show();
         }.bind(this);
