@@ -1,5 +1,6 @@
 /*
  *     Copyright (c) 2016-2017 CoNWeT Lab., Universidad Politécnica de Madrid
+ *     Copyright (c) 2018 Future Internet Consulting and Development Solutions S.L.
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -19,7 +20,7 @@
  *
  */
 
-/* globals encodeURIComponent, StyledElements, Wirecloud */
+/* globals encodeURIComponent, StyledElements, URLify, Wirecloud */
 
 
 (function (ns, se, utils) {
@@ -217,7 +218,7 @@
         });
 
         this.contextManager.modify({
-            title: data.title != null && data.title.trim() != "" ? data.title : data.name,
+            title: data.title != null && data.title.trim() !== "" ? data.title : data.name,
             name: data.name,
             owner: data.owner,
             description: data.description,
@@ -453,9 +454,9 @@
          * @returns {Wirecloud.Task}
          */
         remove: function remove() {
-            return Wirecloud.removeWorkspace(this).then(function () {
+            return Wirecloud.removeWorkspace(this).then(() => {
                 this.dispatchEvent('remove');
-            }.bind(this));
+            });
         },
 
         /**
