@@ -1,5 +1,5 @@
 /*
- *     Copyright (c) 2008-2013 CoNWeT Lab., Universidad Politécnica de Madrid
+ *     Copyright (c) 2008-2015 CoNWeT Lab., Universidad Politécnica de Madrid
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -19,9 +19,10 @@
  *
  */
 
-/*global gettext, IWidget, Wirecloud */
+/* globals Wirecloud */
 
-(function () {
+
+(function (utils) {
 
     "use strict";
 
@@ -35,14 +36,13 @@
      * @param horizontalMargin horizontal margin between iwidgets in pixels
      * @param scrollbarSpace   space reserved for the right scroll bar in pixels
      */
-    function ColumnLayout(dragboard, columns, cellHeight, verticalMargin, horizontalMargin, scrollbarSpace) {
+    var ColumnLayout = function ColumnLayout(dragboard, columns, cellHeight, verticalMargin, horizontalMargin, scrollbarSpace) {
         if (arguments.length === 0) {
             return; // Allow empty constructor (allowing hierarchy)
         }
 
         this.initialized = false;
-        this.shadowMatrix = null;    // Temporal matrix of iwidgets used for D&D
-        this.shadowPositions = null;
+        this._buffers = {"base": {}};
         this.columns = columns;
         this.cellHeight = cellHeight;
 
@@ -62,12 +62,15 @@
             this.rightMargin = Math.floor(horizontalMargin / 2) + 1;
         }
 
-        this._clearMatrix();         // Matrix of iwidgets
+        this._clearMatrix();         // Matrix of iWidgets
+        this._buffers.base = {
+            matrix: this.matrix,
+        };
         this.dragboardCursor = null;
         this.iwidgetToMove = null;
 
         Wirecloud.ui.DragboardLayout.call(this, dragboard, scrollbarSpace);
-    }
+    };
 
     /*
      * ColumnLayout extends Wirecloud.ui.DragboardLayout
@@ -131,38 +134,38 @@
         return cells * (100 / this.columns);
     };
 
-    ColumnLayout.prototype.adaptColumnOffset = function (pixels) {
-        var halfColumnWidth = Math.floor(this.fromHCellsToPixels(1) / 2);
-        var offsetInLU = Math.floor(this.fromPixelsToHCells(pixels - this.leftMargin + halfColumnWidth));
-        var offsetInPixels = this.fromHCellsToPixels(offsetInLU) + this.leftMargin;
-        return new Wirecloud.ui.MultiValuedSize(offsetInPixels, offsetInLU);
+    ColumnLayout.prototype.adaptColumnOffset = function adaptColumnOffset(size) {
+        var offsetInLU, pixels, parsedSize;
+
+        parsedSize = this.parseSize(size);
+        if (parsedSize[1] === 'cells') {
+            offsetInLU = Math.round(parsedSize[0]);
+        } else {
+            if (parsedSize[1] === '%') {
+                pixels = Math.round((parsedSize[0] * this.getWidth()) / 100);
+            } else {
+                pixels = parsedSize[0];
+            }
+            offsetInLU = Math.round(this.fromPixelsToHCells(pixels - this.leftMargin));
+        }
+        return new Wirecloud.ui.MultiValuedSize(this.getColumnOffset(offsetInLU), offsetInLU);
     };
 
-    ColumnLayout.prototype.adaptRowOffset = function (pixels) {
-        var halfRowHeight = Math.floor(this.fromVCellsToPixels(1) / 2);
-        var offsetInLU = Math.floor(this.fromPixelsToVCells(pixels - this.topMargin + halfRowHeight));
-        var offsetInPixels = this.fromVCellsToPixels(offsetInLU) + this.topMargin;
-        return new Wirecloud.ui.MultiValuedSize(offsetInPixels, offsetInLU);
-    };
+    ColumnLayout.prototype.adaptRowOffset = function adaptRowOffset(size) {
+        var offsetInLU, pixels, parsedSize;
 
-    ColumnLayout.prototype.adaptHeight = function (contentHeight, fullSize, oldLayout) {
-        oldLayout = oldLayout ? oldLayout : this;
-        fullSize = oldLayout.padHeight(fullSize);
-
-        var paddedFullSizeInCells = Math.ceil(this.fromPixelsToVCells(fullSize));
-        var paddedFullSize = this.fromVCellsToPixels(paddedFullSizeInCells);
-
-        return new Wirecloud.ui.MultiValuedSize(contentHeight + (paddedFullSize - fullSize), paddedFullSizeInCells);
-    };
-
-    ColumnLayout.prototype.adaptWidth = function (contentWidth, fullSize, oldLayout) {
-        oldLayout = oldLayout ? oldLayout : this;
-        fullSize = oldLayout.padWidth(fullSize);
-
-        var paddedFullSizeInCells = Math.ceil(this.fromPixelsToHCells(fullSize));
-        var paddedFullSize = this.fromHCellsToPixels(paddedFullSizeInCells);
-
-        return new Wirecloud.ui.MultiValuedSize(contentWidth + (paddedFullSize - fullSize), paddedFullSizeInCells);
+        parsedSize = this.parseSize(size);
+        if (parsedSize[1] === 'cells') {
+            offsetInLU = Math.round(parsedSize[0]);
+        } else {
+            if (parsedSize[1] === '%') {
+                pixels = Math.round((parsedSize[0] * this.getWidth()) / 100);
+            } else {
+                pixels = parsedSize[0];
+            }
+            offsetInLU = Math.round(this.fromPixelsToVCells(pixels - this.topMargin));
+        }
+        return new Wirecloud.ui.MultiValuedSize(this.getRowOffset(offsetInLU), offsetInLU);
     };
 
     ColumnLayout.prototype.padWidth = function (width) {
@@ -183,24 +186,25 @@
         return this.dragboardTopMargin + this.fromVCellsToPixels(row) + this.topMargin;
     };
 
-    ColumnLayout.prototype._getPositionOn = function (_matrix, widget) {
-        if (_matrix === this.matrix) {
-            return widget.getPosition();
+    ColumnLayout.prototype._getPositionOn = function _getPositionOn(buffer, widget) {
+        if (buffer === this.matrix || buffer === "base") {
+            return widget.position;
         } else {
-            return this.shadowPositions[widget.code];
+            return this._buffers[buffer].positions[widget.id];
         }
     };
 
-    ColumnLayout.prototype._setPositionOn = function (_matrix, widget, position) {
-        if (_matrix === this.matrix) {
+    ColumnLayout.prototype._setPositionOn = function _setPositionOn(buffer, widget, position) {
+        if (buffer === this.matrix || buffer === "base") {
             widget.setPosition(position);
         } else {
-            this.shadowPositions[widget.code] = position;
+            this._buffers[buffer].positions[widget.id] = position;
         }
     };
 
     ColumnLayout.prototype._clearMatrix = function () {
         this.matrix = [];
+        this._buffers.base.matrix = this.matrix;
 
         for (var x = 0; x < this.getColumns(); x++) {
             this.matrix[x] = [];
@@ -210,6 +214,10 @@
     ColumnLayout.prototype._hasSpaceFor = function (_matrix, positionX, positionY, width, height) {
         var x, y;
 
+        if (typeof _matrix === "string") {
+            _matrix = this._buffers[_matrix].matrix;
+        }
+
         for (x = 0; x < width; x++) {
             for (y = 0; y < height; y++) {
                 if (_matrix[positionX + x][positionY + y] != null) {
@@ -221,24 +229,32 @@
         return true;
     };
 
-    ColumnLayout.prototype._reserveSpace = function (_matrix, iWidget) {
+    ColumnLayout.prototype._reserveSpace = function (_matrix, widget) {
         var x, y;
-        var position = this._getPositionOn(_matrix, iWidget);
-        var width = iWidget.getWidth();
-        var height = iWidget.getHeight();
+        var position = this._getPositionOn(_matrix, widget);
+        var width = widget.shape.width;
+        var height = widget.shape.height;
+
+        if (typeof _matrix === "string") {
+            _matrix = this._buffers[_matrix].matrix;
+        }
 
         for (x = 0; x < width; x++) {
             for (y = 0; y < height; y++) {
-                _matrix[position.x + x][position.y + y] = iWidget;
+                _matrix[position.x + x][position.y + y] = widget;
             }
         }
     };
 
-    ColumnLayout.prototype._clearSpace = function (_matrix, iWidget) {
+    ColumnLayout.prototype._clearSpace = function (_matrix, widget) {
         var x, y;
-        var position = this._getPositionOn(_matrix, iWidget);
-        var width = iWidget.getWidth();
-        var height = iWidget.getHeight();
+        var position = this._getPositionOn(_matrix, widget);
+        var width = widget.shape.width;
+        var height = widget.shape.height;
+
+        if (typeof _matrix === "string") {
+            _matrix = this._buffers[_matrix].matrix;
+        }
 
         for (x = 0; x < width; x++) {
             for (y = 0; y < height; y++) {
@@ -249,7 +265,7 @@
         this._compressColumns(_matrix, position.x, width);
     };
 
-    ColumnLayout.prototype._compressColumns = function(_matrix, x, width) {
+    ColumnLayout.prototype._compressColumns = function _compressColumns(_matrix, x, width) {
         var i, y;
 
         for (i = 0; i < width; i++) {
@@ -262,26 +278,27 @@
         }
     };
 
-    ColumnLayout.prototype._searchInsertPoint = function (_matrix, x, y, width, height) {
+    ColumnLayout.prototype._searchInsertPoint = function _searchInsertPoint(_matrix, x, y, width, height) {
         return y;
     };
 
-    ColumnLayout.prototype._moveSpaceDown = function (_matrix, iWidget, offsetY) {
-        var affectedIWidgets, position, finalPosition, edgeY, iwidget, x, y, key;
+    ColumnLayout.prototype._moveSpaceDown = function _moveSpaceDown(buffer, widget, offsetY) {
+        var affectedIWidgets, position, finalPosition, edgeY, curwidget, x, y, key, matrix;
 
         affectedIWidgets = {};
-        position = this._getPositionOn(_matrix, iWidget);
+        matrix = this._buffers[buffer].matrix;
+        position = this._getPositionOn(buffer, widget);
         finalPosition = new Wirecloud.DragboardPosition(position.x, position.y + offsetY);
 
-        edgeY = position.y + iWidget.getHeight();
+        edgeY = position.y + widget.shape.height;
 
         // Search affected widgets
         // TODO move widgets according to the biggest offset for optimizing
-        for (x = 0; x < iWidget.getWidth(); x++) {
+        for (x = 0; x < widget.shape.width; x++) {
             for (y = 0; y < offsetY; y++) {
-                iwidget = _matrix[position.x + x][edgeY + y];
-                if (iwidget != null) {
-                    affectedIWidgets[iwidget.code] = offsetY - y; // calculate the offset for this iwidget
+                curwidget = matrix[position.x + x][edgeY + y];
+                if (curwidget != null) {
+                    affectedIWidgets[curwidget.id] = offsetY - y; // calculate the offset for this widget
                     break; // continue whit the next column
                 }
             }
@@ -289,25 +306,29 @@
 
         // Move affected widgets instances
         for (key in affectedIWidgets) {
-            iwidget = this.iWidgets[key];
-            this._moveSpaceDown(_matrix, iwidget, affectedIWidgets[key]);
+            curwidget = this.widgets[key];
+            this._moveSpaceDown(buffer, curwidget, affectedIWidgets[key]);
         }
 
         // Move the widget
-        this._clearSpace(_matrix, iWidget);
-        this._setPositionOn(_matrix, iWidget, finalPosition);
-        this._reserveSpace(_matrix, iWidget);
+        this._clearSpace(buffer, widget);
+        this._setPositionOn(buffer, widget, finalPosition);
+        this._reserveSpace(buffer, widget);
     };
 
-    ColumnLayout.prototype._moveSpaceUp = function (_matrix, iWidget) {
-        var position, edgeY, offsetY, affectedIWidgets, finalPosition, iwidget,
-            x, y, columnsize, key;
+    /**
+     * @returns Returns true if the widget position has been modified
+     */
+    ColumnLayout.prototype._moveSpaceUp = function (buffer, widget) {
+        var position, edgeY, offsetY, affectedIWidgets, finalPosition, curWidget,
+            x, y, columnsize, key, matrix;
 
-        position = this._getPositionOn(_matrix, iWidget);
-        edgeY = position.y + iWidget.getHeight();
+        matrix = this._buffers[buffer].matrix;
+        position = this._getPositionOn(buffer, widget);
+        edgeY = position.y + widget.shape.height;
 
         offsetY = 1;
-        while (((position.y - offsetY) >= 0) && this._hasSpaceFor(_matrix, position.x, position.y - offsetY, iWidget.getWidth(), 1)) {
+        while (((position.y - offsetY) >= 0) && this._hasSpaceFor(matrix, position.x, position.y - offsetY, widget.shape.width, 1)) {
             offsetY += 1;
         }
         offsetY -= 1;
@@ -318,44 +339,43 @@
 
             // Search affected widgets
             // TODO move the topmost widget for optimizing
-            for (x = 0; x < iWidget.getWidth(); x++) {
-                columnsize = _matrix[position.x + x].length;
+            for (x = 0; x < widget.shape.width; x++) {
+                columnsize = matrix[position.x + x].length;
                 for (y = edgeY; y < columnsize; y++) {
-                    iwidget = _matrix[position.x + x][y];
-                    if (iwidget != null) {
-                        affectedIWidgets[iwidget.code] = iwidget;
+                    curWidget = matrix[position.x + x][y];
+                    if (curWidget != null) {
+                        affectedIWidgets[curWidget.id] = curWidget;
                         break; // continue whit the next column
                     }
                 }
             }
 
             // Move the representation of the widget
-            this._clearSpace(_matrix, iWidget);
-            this._setPositionOn(_matrix, iWidget, finalPosition);
-            this._reserveSpace(_matrix, iWidget);
+            this._clearSpace(buffer, widget);
+            this._setPositionOn(buffer, widget, finalPosition);
+            this._reserveSpace(buffer, widget);
 
             // Move affected widgets instances
             for (key in affectedIWidgets) {
-                this._moveSpaceUp(_matrix, affectedIWidgets[key]);
+                this._moveSpaceUp(buffer, affectedIWidgets[key]);
             }
 
-            //return true if the iwidget position has been modified
             return true;
         }
         return false;
     };
 
-    ColumnLayout.prototype._removeFromMatrix = function (_matrix, iWidget) {
-        this._clearSpace(_matrix, iWidget);
+    ColumnLayout.prototype._removeFromMatrix = function (_matrix, widget) {
+        this._clearSpace(_matrix, widget);
         return false;
     };
 
-    ColumnLayout.prototype._reserveSpace2 = function (_matrix, iWidget, positionX, positionY, width, height) {
+    ColumnLayout.prototype._reserveSpace2 = function (_matrix, widget, positionX, positionY, width, height) {
         var x, y;
 
         for (x = 0; x < width; x++) {
             for (y = 0; y < height; y++) {
-                _matrix[positionX + x][positionY + y] = iWidget;
+                _matrix[positionX + x][positionY + y] = widget;
             }
         }
     };
@@ -372,10 +392,10 @@
         this._compressColumns(_matrix, positionX, width);
     };
 
-    ColumnLayout.prototype._notifyResizeEvent = function (iWidget, oldWidth, oldHeight, newWidth, newHeight, resizeLeftSide, persist) {
+    ColumnLayout.prototype._notifyResizeEvent = function (widget, oldWidth, oldHeight, newWidth, newHeight, resizeLeftSide, persist) {
         var x, y;
         var step2Width = oldWidth; // default value, used when the igdaget's width doesn't change
-        var position = iWidget.getPosition();
+        var position = widget.position;
         var step2X, iWidgetToMove, finalYPos, widthDiff;
         step2X = position.x;
 
@@ -393,7 +413,7 @@
                     for (y = 0; y < newHeight; ++y) {
                         iWidgetToMove = this.matrix[x][position.y + y];
                         if (iWidgetToMove != null) {
-                            this._moveSpaceDown(this.matrix, iWidgetToMove, finalYPos - iWidgetToMove.position.y);
+                            this._moveSpaceDown("base", iWidgetToMove, finalYPos - iWidgetToMove.position.y);
                             break; // Continue with the next column
                         }
                     }
@@ -401,12 +421,10 @@
 
                 // Move the widget
                 position.x -= widthDiff;
-                if (persist) {
-                    iWidget.setPosition(position);
-                }
+                widget.setPosition(position);
 
                 // Reserve the new space
-                this._reserveSpace2(this.matrix, iWidget,
+                this._reserveSpace2(this.matrix, widget,
                                                  position.x, position.y,
                                                  widthDiff, newHeight);
             } else {
@@ -415,14 +433,14 @@
                     for (y = 0; y < newHeight; ++y) {
                         iWidgetToMove = this.matrix[x][position.y + y];
                         if (iWidgetToMove != null) {
-                            this._moveSpaceDown(this.matrix, iWidgetToMove, finalYPos - iWidgetToMove.position.y);
+                            this._moveSpaceDown("base", iWidgetToMove, finalYPos - iWidgetToMove.position.y);
                             break; // Continue with the next column
                         }
                     }
                 }
 
                 // Reserve this space
-                this._reserveSpace2(this.matrix, iWidget,
+                this._reserveSpace2(this.matrix, widget,
                                                  position.x + oldWidth, position.y,
                                                  newWidth - oldWidth, newHeight);
             }
@@ -439,9 +457,7 @@
 
                 // Move the widget
                 position.x += widthDiff;
-                if (persist) {
-                    iWidget.setPosition(position);
-                }
+                widget.setPosition(position);
 
                 step2X = position.x;
             } else {
@@ -458,13 +474,13 @@
             for (y = position.y + oldHeight; y < limitY; y++) {
                 for (x = step2X; x < limitX; x++) {
                     if (this.matrix[x][y] != null) {
-                        this._moveSpaceDown(this.matrix, this.matrix[x][y], limitY - y);
+                        this._moveSpaceDown("base", this.matrix[x][y], limitY - y);
                     }
                 }
             }
 
             // Reserve Space
-            this._reserveSpace2(this.matrix, iWidget, step2X, position.y + oldHeight, step2Width, newHeight - oldHeight);
+            this._reserveSpace2(this.matrix, widget, step2X, position.y + oldHeight, step2Width, newHeight - oldHeight);
         } else if (newHeight < oldHeight) {
             // Clear freed space
             this._clearSpace2(this.matrix, step2X, position.y + newHeight, step2Width, oldHeight - newHeight);
@@ -472,28 +488,36 @@
 
         this._notifyWindowResizeEvent(true, true); // TODO
         if (persist) {
-            this.dragboard._commitChanges(); // FIXME
+            this.dragboard.update(); // FIXME
         }
     };
 
-    ColumnLayout.prototype._insertAt = function (iWidget, x, y) {
+    /**
+     * @returns returns if any widget's position has been modified
+     */
+    ColumnLayout.prototype._insertAt = function _insertAt(widget, x, y, buffer) {
         var newPosition = new Wirecloud.DragboardPosition(x > 0 ? x : 0, y > 0 ? y : 0);
 
         // Move other instances
-        var affectedwidget, offset, affectedY;
+        var affectedwidget, offset, affectedY, matrix;
         var affectedWidgets = false;
-        var lastX = newPosition.x + iWidget.getWidth();
-        var lastY = newPosition.y + iWidget.getHeight();
+        var lastX = newPosition.x + widget.shape.width;
+        var lastY = newPosition.y + widget.shape.height;
+
+        if (buffer == null) {
+            buffer = "base";
+        }
+        matrix = this._buffers[buffer].matrix;
 
         for (x = newPosition.x; x < lastX; x++) {
             for (y = newPosition.y; y < lastY; y++) {
-                affectedwidget = this.matrix[x][y];
+                affectedwidget = matrix[x][y];
                 if (affectedwidget != null) {
                     // only move the widget if we didn't move it before
-                    affectedY = affectedwidget.getPosition().y;
-                    // y + iWidget.getHeight() - affectedY - (newPosition.y - y);
+                    affectedY = this._getPositionOn(buffer, affectedwidget).y;
+                    // y + widget.shape.height - affectedY - (newPosition.y - y);
                     offset = lastY - affectedY;
-                    this._moveSpaceDown(this.matrix, affectedwidget,  offset);
+                    this._moveSpaceDown(buffer, affectedwidget, offset);
                     // move only the topmost widget in the column
                     affectedWidgets = true;
                     break;
@@ -502,20 +526,19 @@
         }
 
         // Change Widget instance position (insert it)
-        iWidget.setPosition(newPosition);
+        this._setPositionOn(buffer, widget, newPosition);
 
-        this._reserveSpace(this.matrix, iWidget);
+        this._reserveSpace(buffer, widget);
 
-        //returns if any widget's position has been modified
         return affectedWidgets;
     };
 
     ColumnLayout.prototype._searchFreeSpace = function (width, height) {
         var positionX = 0, positionY = 0;
-        var columns = this.getColumns() - width + 1;
+        var maxX = this.columns - width;
 
         for (positionY = 0; true ; positionY++) {
-            for (positionX = 0; positionX < columns; positionX++) {
+            for (positionX = 0; positionX <= maxX; positionX++) {
                 if (this._hasSpaceFor(this.matrix, positionX, positionY, width, height)) {
                     return new Wirecloud.DragboardPosition(positionX, positionY);
                 }
@@ -524,28 +547,28 @@
     };
 
     ColumnLayout.prototype.initialize = function () {
-        var iWidget, i, key, position, iWidgetsToReinsert = [];
+        var widget, i, key, position, iWidgetsToReinsert = [];
 
         this._clearMatrix();
 
         // Insert iwidgets
-        for (key in this.iWidgets) {
-            iWidget = this.iWidgets[key];
+        for (key in this.widgets) {
+            widget = this.widgets[key];
 
-            position = iWidget.getPosition();
+            position = widget.position;
 
-            iWidget.paint(true);
+            widget.repaint();
 
-            if (iWidget.getWidth() > this.getColumns()) {
-                iWidget.contentWidth = this.getColumns();
+            if (widget.shape.width > this.getColumns()) {
+                widget.contentWidth = this.getColumns();
             }
 
-            if (iWidget.getWidth() + position.x > this.getColumns()) {
-                iWidgetsToReinsert.push(iWidget);
-            } else if (this._hasSpaceFor(this.matrix, position.x, position.y, iWidget.getWidth(), iWidget.getHeight())) {
-                this._reserveSpace(this.matrix, iWidget);
+            if (widget.shape.width + position.x > this.getColumns()) {
+                iWidgetsToReinsert.push(widget);
+            } else if (this._hasSpaceFor(this.matrix, position.x, position.y, widget.shape.width, widget.shape.height)) {
+                this._reserveSpace(this.matrix, widget);
             } else {
-                iWidgetsToReinsert.push(iWidget);
+                iWidgetsToReinsert.push(widget);
             }
         }
 
@@ -553,8 +576,8 @@
         if (iWidgetsToReinsert.length > 0) {
             // Reinsert the iwidgets that didn't fit in their positions
             for (i = 0; i < iWidgetsToReinsert.length; i++) {
-                position = this._searchFreeSpace(iWidgetsToReinsert[i].getWidth(),
-                                                 iWidgetsToReinsert[i].getHeight());
+                position = this._searchFreeSpace(iWidgetsToReinsert[i].shape.width,
+                                                 iWidgetsToReinsert[i].shape.height);
                 iWidgetsToReinsert[i].setPosition(position);
                 this._reserveSpace(this.matrix, iWidgetsToReinsert[i]);
             }
@@ -576,61 +599,58 @@
     };
 
     /**
-     * Inserts the given iWidget into this layout.
+     * Inserts the given widget into this layout.
      *
-     * @param iWidget the iWidget to insert in this layout
+     * @param widget the widget to insert in this layout
      * @param affectsDragboard if true, the dragboard associated to this layout will be notified
      * @return whether any widget's position has been modified
      */
-    ColumnLayout.prototype.addIWidget = function (iWidget, affectsDragboard) {
+    ColumnLayout.prototype.addWidget = function addWidget(widget, affectsDragboard) {
         var affectedWidgets = false;
 
-        Wirecloud.ui.DragboardLayout.prototype.addIWidget.call(this, iWidget, affectsDragboard);
+        Wirecloud.ui.DragboardLayout.prototype.addWidget.call(this, widget, affectsDragboard);
 
         if (!this.initialized) {
             return;
         }
 
-        if (iWidget.getWidth() > this.getColumns()) {
-            iWidget.contentWidth = this.getColumns();
-            if (iWidget.isVisible()) {
-                iWidget._recomputeSize(true);
-            }
+        if (widget.shape.width > this.getColumns()) {
+            widget.setShape({width: this.getColumns()});
+            // TODO: persist
         }
 
-        var position = iWidget.getPosition();
+        var position = widget.position;
         if (position) {
-            var diff = iWidget.getWidth() + position.x - this.getColumns();
+            var diff = widget.shape.width + position.x - this.getColumns();
             if (diff > 0) {
                 position.x -= diff;
             }
 
             // Insert it. Returns if there are any affected widget
-            affectedWidgets = this._insertAt(iWidget, position.x, position.y);
+            affectedWidgets = this._insertAt(widget, position.x, position.y, "base");
         } else {
             // Search a position for the widget
-            position = this._searchFreeSpace(iWidget.getWidth(), iWidget.getHeight());
-            iWidget.setPosition(position);
+            position = this._searchFreeSpace(widget.shape.width, widget.shape.height);
+            widget.setPosition(position);
 
             // Reserve the cells for the widget instance
-            this._reserveSpace(this.matrix, iWidget);
+            this._reserveSpace(this.matrix, widget);
         }
 
-        this._adaptIWidget(iWidget);
+        this._adaptIWidget(widget);
         return affectedWidgets;
     };
 
-    //Returns if any widget's position has been modified
-    ColumnLayout.prototype.removeIWidget = function (iWidget, affectsDragboard) {
+    ColumnLayout.prototype.removeWidget = function removeWidget(widget, affectsDragboard) {
         var affectedWidgets;
 
-        affectedWidgets = this._removeFromMatrix(this.matrix, iWidget);
-        Wirecloud.ui.DragboardLayout.prototype.removeIWidget.call(this, iWidget, affectsDragboard);
+        affectedWidgets = this._removeFromMatrix("base", widget);
+        Wirecloud.ui.DragboardLayout.prototype.removeWidget.call(this, widget, affectsDragboard);
         return affectedWidgets;
     };
 
     ColumnLayout.prototype.moveTo = function (destLayout) {
-        var movedWidgets, orderedWidgets, x, y, i, columns, rows, iWidget;
+        var movedWidgets, orderedWidgets, x, y, i, columns, rows, widget;
 
         /*
          * Always use ColumnLayout._removeFromMatrix for removing widgets
@@ -644,64 +664,85 @@
         rows = this.getRows();
         for (y = 0; y < rows; y += 1) {
             for (x = 0; x < columns; x += 1) {
-                var iwidget = this.matrix[x][y];
-                if (iwidget != null && !(iwidget.id in movedWidgets)) {
-                    orderedWidgets.push(iwidget);
-                    movedWidgets[iwidget.id] = true;
+                widget = this.matrix[x][y];
+                if (widget != null && !(widget.id in movedWidgets)) {
+                    orderedWidgets.push(widget);
+                    movedWidgets[widget.id] = true;
                 }
             }
         }
 
         for (i = 0; i < orderedWidgets.length; i += 1) {
-            iWidget = orderedWidgets[i];
-            iWidget.moveToLayout(destLayout);
+            widget = orderedWidgets[i];
+            widget.moveToLayout(destLayout);
         }
 
         /* Restore _removeFromMatrix */
         delete this._removeFromMatrix;
     };
 
-    ColumnLayout.prototype.initializeMove = function (iwidget, draggable) {
-        var msg, key, i, lastWidget, lastY, tmp;
+    ColumnLayout.prototype._cloneMatrix = function _cloneMatrix(matrix) {
+        var i, cloned_matrix = [];
+
+        for (i = 0; i < this.columns; i++) {
+            cloned_matrix[i] = Wirecloud.Utils.clone(matrix[i]);
+        }
+
+        return cloned_matrix;
+    };
+
+    ColumnLayout.prototype._clonePositions = function _clonePositions(buffer) {
+        var key, positions = {};
+
+        for (key in this.widgets) {
+            positions[key] = utils.clone(this._getPositionOn(buffer, this.widgets[key]));
+        }
+
+        return positions;
+    };
+
+    ColumnLayout.prototype.initializeMove = function (widget, draggable) {
+        var msg, i, lastWidget, lastY, tmp;
 
         draggable = draggable || null; // default value of draggable argument
 
+        if (!(widget instanceof Wirecloud.ui.WidgetView)) {
+            throw new TypeError("widget must be an WidgetView instance");
+        }
+
         // Check for pendings moves
-        if (this.iwidgetToMove !== null) {
-            msg = gettext("There was a pending move that was cancelled because initializedMove function was called before it was finished.");
+        if (this.iwidgetToMove != null) {
+            msg = "Dragboard: There was a pending move that was cancelled because initializedMove function was called before it was finished.";
             Wirecloud.GlobalLogManager.log(msg, Wirecloud.constants.LOGGING.WARN_MSG);
             this.cancelMove();
         }
 
-        this.iwidgetToMove = iwidget;
+        this.iwidgetToMove = widget;
 
         // Make a copy of the positions of the widgets
-        this.shadowPositions = [];
-
-        for (key in this.iWidgets) {
-            this.shadowPositions[key] = this.iWidgets[key].getPosition().clone();
-        }
+        this._buffers.backup = {};
+        this._buffers.backup.positions = this._clonePositions(this.matrix);
+        this._buffers.shadow = {
+            "matrix": this.matrix
+        };
 
         // Shadow matrix = current matrix without the widget to move
         // Initialize shadow matrix and searchInsertPointCache
         lastY = 0;
-        this.shadowMatrix = [];
-        this.searchInsertPointCache = [];
-        for (i = 0; i < this.columns; i++) {
-            this.searchInsertPointCache[i] = [];
-            this.shadowMatrix[i] = Wirecloud.Utils.clone(this.matrix[i]);
-        }
-        this._removeFromMatrix(this.shadowMatrix, iwidget);
+        this._buffers.backup.matrix = this._cloneMatrix(this.matrix);
+        this._removeFromMatrix("backup", widget);
 
+        this.searchInsertPointCache = [];
         // search bottommost row
         for (i = 0; i < this.columns; i++) {
+            this.searchInsertPointCache[i] = [];
             lastWidget = this.matrix[i][this.matrix[i].length - 1];
 
             if (!lastWidget) {
                 continue;
             }
 
-            tmp = lastWidget.getPosition().y + lastWidget.getHeight();
+            tmp = lastWidget.position.y + lastWidget.shape.height;
             if (tmp > lastY) {
                 lastY = tmp;
             }
@@ -709,9 +750,7 @@
         this.searchInsertPointYLimit = lastY + 1;
 
         // Create dragboard cursor
-        this.dragboardCursor = new Wirecloud.ui.DragboardCursor(iwidget);
-        this.dragboardCursor.paint(this.dragboard.dragboardElement);
-        this._reserveSpace(this.matrix, this.dragboardCursor);
+        this.dragboardCursor = new Wirecloud.ui.DragboardCursor(widget);
 
         if (draggable) {
             draggable.setXOffset(this.fromHCellsToPixels(1) / 2);
@@ -719,111 +758,107 @@
         }
     };
 
-    ColumnLayout.prototype._destroyCursor = function (clearSpace) {
+    ColumnLayout.prototype._destroyCursor = function _destroyCursor() {
         if (this.dragboardCursor !== null) {
-            if (clearSpace) {
-                this._removeFromMatrix(this.matrix, this.dragboardCursor);
-            }
             this.dragboardCursor.destroy();
             this.dragboardCursor = null;
         }
     };
 
     ColumnLayout.prototype.disableCursor = function () {
-        this._destroyCursor(true);
+        this._destroyCursor();
     };
 
-    /**
-     * @private
-     */
-    ColumnLayout.prototype._restorePositions = function () {
-        this._clearMatrix();
+    ColumnLayout.prototype._setPositions = function _setPositions() {
+        var key, curIWidget;
 
-        var key;
-        for (key in this.iWidgets) {
-            var curIWidget = this.iWidgets[key];
+        for (key in this.widgets) {
+            curIWidget = this.widgets[key];
             if (curIWidget !== this.iwidgetToMove) {
-                curIWidget.setPosition(this._getPositionOn(this._matrix, curIWidget));
-
-                this._reserveSpace(this.matrix, curIWidget);
+                curIWidget.setPosition(this._getPositionOn("shadow", curIWidget));
             }
         }
+
+        this.dragboardCursor.setPosition(this._getPositionOn("shadow", this.dragboardCursor));
     };
 
-    ColumnLayout.prototype.moveTemporally = function (x, y) {
-        if (!(this.iwidgetToMove instanceof IWidget)) {
-            var msg = gettext("Dragboard: You must call initializeMove function before calling to this function (moveTemporally).");
+    ColumnLayout.prototype.moveTemporally = function moveTemporally(x, y) {
+        if (this.iwidgetToMove == null) {
+            var msg = "Dragboard: You must call initializeMove function before calling to this function (moveTemporally).";
             Wirecloud.GlobalLogManager.log(msg, Wirecloud.constants.LOGGING.WARN_MSG);
             return;
         }
 
-        var maxX = this.getColumns() - this.iwidgetToMove.getWidth();
+        var maxX = this.getColumns() - this.iwidgetToMove.shape.width;
         if (x > maxX) {
             x = maxX;
         }
 
         if (this.dragboardCursor != null) {
-            var cursorpos = this.dragboardCursor.getPosition();
+            var cursorpos = this.dragboardCursor.position;
 
             if ((cursorpos.y !== y) || (cursorpos.x !== x)) {
-                this._restorePositions();
+                this._buffers.shadow.positions = this._clonePositions("backup");
+                this._buffers.shadow.matrix = this._cloneMatrix(this._buffers.backup.matrix);
 
                 // Change cursor position
-                //this._removeFromMatrix(this.matrix, this.dragboardCursor); Done at _restorePositions
-                this._insertAt(this.dragboardCursor, x, y);
+                this._insertAt(this.dragboardCursor, x, y, "shadow");
+                this._setPositions();
             }
         } else {
+            this._buffers.shadow.positions = this._clonePositions("backup");
+            this._buffers.shadow.matrix = this._cloneMatrix(this._buffers.backup.matrix);
+
             this.dragboardCursor = new Wirecloud.ui.DragboardCursor(this.iwidgetToMove);
-            this.dragboardCursor.paint(this.dragboard.dragboardElement);
-            this._insertAt(this.dragboardCursor, x, y);
+            this._insertAt(this.dragboardCursor, x, y, "shadow");
         }
     };
 
     ColumnLayout.prototype.cancelMove = function () {
-        if (!(this.iwidgetToMove instanceof IWidget)) {
-            var msg = gettext("Trying to cancel an inexistant temporal move.");
+        if (this.iwidgetToMove == null) {
+            var msg = "Dragboard: Trying to cancel an inexistant temporal move.";
             Wirecloud.GlobalLogManager.log(msg, Wirecloud.constants.LOGGING.WARN_MSG);
             return;
         }
 
-        this._destroyCursor(true);
-        var position = this.iwidgetToMove.getPosition();
-        this._insertAt(this.iwidgetToMove, position.x, position.y);
-        this.shadowMatrix = null;
+        this._destroyCursor();
+
+        for (var key in this.widgets) {
+            this.widgets[key].setPosition(this.widgets[key].position);
+        }
         this.iwidgetToMove = null;
         this.dragboardCursor = null;
     };
 
     ColumnLayout.prototype.acceptMove = function () {
-        if (!(this.iwidgetToMove instanceof IWidget)) {
-            var msg = gettext("Function acceptMove called when there is not an started iwidget move.");
+        if (this.iwidgetToMove == null) {
+            var msg = "Dragboard: Function acceptMove called when there is not an started widget move.";
             Wirecloud.GlobalLogManager.log(msg, Wirecloud.constants.LOGGING.WARN_MSG);
             return;
         }
 
-        var oldposition = this.iwidgetToMove.getPosition();
-        var newposition = this.dragboardCursor.getPosition();
-        this._destroyCursor(false);
+        var oldposition = this.iwidgetToMove.position;
+        var newposition = this.dragboardCursor.position;
+        this._destroyCursor();
 
-        // Needed to force repaint of the iwidget at the correct position
+        // Needed to force repaint of the widget at the correct position
         this.iwidgetToMove.setPosition(newposition);
 
-        // Needed to overwriting the cursor cells
-        this._reserveSpace(this.matrix, this.iwidgetToMove);
-
         // Update iwidgets positions in persistence
         if (oldposition.y !== newposition.y || oldposition.x !== newposition.x) {
-            this.dragboard._commitChanges();
+            this.matrix = this._buffers.shadow.matrix;
+            this._buffers.base.matrix = this.matrix;
+            this._reserveSpace("base", this.iwidgetToMove);
+            this.dragboard.update();
         }
 
         // This is needed to check if the scrollbar status has changed (visible/hidden)
         this.dragboard._notifyWindowResizeEvent();
 
-        this.shadowMatrix = null;
         this.iwidgetToMove = null;
         this.dragboardCursor = null;
     };
 
     Wirecloud.ui.ColumnLayout = ColumnLayout;
 
-})();
+})(Wirecloud.Utils);
