@@ -1,5 +1,5 @@
 /*
- *     (C) Copyright 2008-2013 Universidad Politécnica de Madrid
+ *     Copyright (c) 2008-2016 CoNWeT Lab., Universidad Politécnica de Madrid
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -19,44 +19,56 @@
  *
  */
 
-/*global gettext, interpolate, Wirecloud*/
+/* globals Wirecloud */
 
-(function () {
+
+(function (utils) {
 
     "use strict";
 
-    var Endpoint = function Endpoint(name, type, friendCode, id) {
-        this.id = id;
+    var Endpoint = function Endpoint() {
+
+        if (!('name' in this)) {
+            Object.defineProperty(this, 'name', {value: null});
+        }
+
+        if (!('friendcode' in this)) {
+            Object.defineProperty(this, 'friendcode', {value: ""});
+        }
+
+        var friendcodeList = this.friendcode !== '' ? this.friendcode.split(/\s+/) : [];
+        Object.defineProperty(this, 'friendcodeList', {value: friendcodeList});
+
+        if (!('label' in this)) {
+            Object.defineProperty(this, 'label', {value: ""});
+        }
+        if (!('description' in this)) {
+            Object.defineProperty(this, 'description', {value: ""});
+        }
+
+        if (!('id' in this)) {
+            Object.defineProperty(this, 'id', {value: null});
+        }
+
     };
 
-    Endpoint.prototype.getId = function getId() {
-        return this.id;
-    };
-
-    Endpoint.prototype.getFinalSlots = function getFinalSlots() {
-        return null;
+    Endpoint.prototype.getReachableEndpoints = function getReachableEndpoints() {
+        return [];
     };
 
     /**
      * Disconnects this <code>Endpoint</code> from all the
      * <code>Endpoints</code> this is connected to.
      */
+    /* istanbul ignore next */
     Endpoint.prototype.fullDisconnect = function fullDisconnect() {
         var funcName = 'fullDisconnect';
-        var msg = gettext("Unimplemented function: %(funcName)s");
-        msg = interpolate(msg, {funcName: funcName}, true);
+        var msg = utils.gettext("Unimplemented function: %(funcName)s");
+        msg = utils.interpolate(msg, {funcName: funcName}, true);
         Wirecloud.GlobalLogManager.log(msg);
         return;
     };
 
-    /**
-     * This method must be called to avoid memory leaks caused by circular
-     * references.
-     */
-    Endpoint.prototype.destroy = function destroy() {
-        this.fullDisconnect();
-    };
-
     Wirecloud.wiring.Endpoint = Endpoint;
 
-})();
+})(Wirecloud.Utils);
