@@ -27,7 +27,7 @@ from lxml import etree
 import django
 from django.contrib.auth.models import Group, User
 from django.core.urlresolvers import reverse
-from django.test import Client
+from django.test import Client, TransactionTestCase
 from django.test.client import RequestFactory
 from django.test.utils import override_settings
 from django.template import TemplateDoesNotExist
@@ -51,10 +51,11 @@ from django.utils.translation import LANGUAGE_SESSION_KEY
 __test__ = False
 
 
-class BasicViewsAPI(WirecloudTestCase):
+class BasicViewsAPI(WirecloudTestCase, TransactionTestCase):
 
     fixtures = ('selenium_test_data', 'user_with_workspaces')
     tags = ('wirecloud-base-views', 'wirecloud-base-views-unit', 'wirecloud-noselenium')
+    use_search_indexes = False
 
     def setUp(self):
         super(BasicViewsAPI, self).setUp()
@@ -66,12 +67,7 @@ class BasicViewsAPI(WirecloudTestCase):
         super(BasicViewsAPI, cls).setUpClass()
         factory = RequestFactory()
         request = factory.get(reverse('login'))
-        if django.VERSION[1] >= 9:
-            # Django 1.9 doesn't force the use of absolute urls for the location header
-            # https://docs.djangoproject.com/en/1.9/releases/1.9/#http-redirects-no-longer-forced-to-absolute-uris
-            cls.login_url = reverse('login')
-        else:
-            cls.login_url = get_absolute_reverse_url('login', request=request)
+        cls.login_url = reverse('login')
 
     def test_workspace_view_redirects_to_login(self):
 
