@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2013 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2013-2016 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -19,27 +19,32 @@
 
 import json
 
-from django.http import HttpResponse
-from django.template import RequestContext
+from django.http import Http404, HttpResponse
+from django.template import TemplateDoesNotExist
 from django.template.loader import get_template
 from django.utils.decorators import method_decorator
 from django.views.decorators.cache import cache_page
 
 from wirecloud.commons.baseviews.resource import Resource
+from wirecloud.commons.utils.encoding import LazyEncoder
+from wirecloud.commons.utils.http import get_absolute_static_url
 from wirecloud.platform.plugins import get_plugins
+from wirecloud.platform.themes import get_theme_metadata
+
 
 _wirecloud_templates = {}
 
+
 def get_templates(view):
 
     global _wirecloud_templates
 
     if view not in _wirecloud_templates:
         plugins = get_plugins()
-        templates = {}
+        templates = []
 
         for plugin in plugins:
-            templates.update(plugin.get_templates(view))
+            templates.extend(plugin.get_templates(view))
 
         _wirecloud_templates[view] = templates
 
@@ -51,11 +56,24 @@ class ThemeEntry(Resource):
     @method_decorator(cache_page(60 * 60 * 24 * 365))
     def read(self, request, name):
 
-        context = RequestContext(request)
-        templates = {}
-        template_descriptions = get_templates('classic')
-        for template_id in template_descriptions:
-            template = get_template(template_descriptions[template_id])
-            templates[template_id] = template.render(context)
+        try:
+            theme_info = get_theme_metadata(name)
+        except ValueError:
+            raise Http404
 
-        return HttpResponse(json.dumps(templates), 'application/json')
+        desc = {
+            "name": name,
+            "label": theme_info.label,
+            "baseurl": get_absolute_static_url("theme/%s/" % name, request),
+            "templates": {}
+        }
+
+        for template_id in get_templates('classic'):
+            template_file = template_id + '.html'
+            try:
+                template = get_template("%s:%s" % (name, template_file))
+            except TemplateDoesNotExist:
+                template = get_template(template_file)
+            desc["templates"][template_id] = template.render({})
+
+        return HttpResponse(json.dumps(desc, cls=LazyEncoder, sort_keys=True), 'application/json')
