@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2011-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2011-2016 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -17,17 +17,19 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with Wirecloud.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import unicode_literals
+
 import os
 import random
 
 from django.core.cache import cache
 from django.db import models
+from django.db.models.signals import pre_delete, post_save
+from django.dispatch import receiver
 from django.utils.encoding import python_2_unicode_compatible
-from django.db.models.signals import post_save
 from django.utils.translation import ugettext as _
 
 from wirecloud.catalogue.models import CatalogueResource
-from wirecloud.commons.models import TransModel
 from wirecloud.commons.utils.wgt import WgtFile
 
 
@@ -45,19 +47,19 @@ class XHTML(models.Model):
     def __str__(self):
         return self.uri
 
-    def get_cache_key(self, domain, mode):
-        version = cache.get('_widget_xhtml_version/' + str(self.id))
+    def get_cache_key(self, domain, mode, theme):
+        version = cache.get('_widget_xhtml_version/%s' % self.id)
         if version is None:
             version = random.randrange(1, 100000)
-            cache.set('_widget_xhtml_version/' + str(self.id), version)
+            cache.set('_widget_xhtml_version/%s' % self.id, version)
 
-        return '_widget_xhtml/' + str(version) + '/' + domain + '/' + str(self.id) + '?mode=' + mode
+        return '_widget_xhtml/%s/%s/%s?mode=%s&theme=%s' % (version, domain, self.id, mode, theme)
 
     def delete(self, *args, **kwargs):
         old_id = self.id
         super(XHTML, self).delete(*args, **kwargs)
         try:
-            cache.incr('_widget_xhtml_version/' + str(old_id))
+            cache.incr('_widget_xhtml_version/%s' % old_id)
         except ValueError:
             pass
 
@@ -77,11 +79,7 @@ class Widget(models.Model):
 
     resource = models.OneToOneField('catalogue.CatalogueResource')
 
-    xhtml = models.ForeignKey(XHTML)
-
-    width = models.IntegerField(_('Width'), default=1)
-    height = models.IntegerField(_('Height'), default=1)
-
+    xhtml = models.ForeignKey(XHTML, on_delete=models.CASCADE)
     objects = WidgetManager()
 
     @property
@@ -98,6 +96,7 @@ class Widget(models.Model):
         return self.resource.is_available_for(user)
 
     def delete(self, *args, **kwargs):
+
         try:
             self.xhtml.delete()
         except XHTML.DoesNotExist:
@@ -110,86 +109,12 @@ class Widget(models.Model):
     def __str__(self):
         return self.uri
 
-    def get_related_preferences(self):
-        return VariableDef.objects.filter(widget=self, aspect='PREF')
-
-    def get_related_properties(self):
-        return VariableDef.objects.filter(widget=self, aspect='PROP')
-
-
-@python_2_unicode_compatible
-class VariableDef(TransModel):
-
-    name = models.CharField(_('Name'), max_length=30)
-    TYPES = (
-        ('N', _('Number')),
-        ('S', _('String')),
-        ('D', _('Date')),
-        ('B', _('Boolean')),
-        ('P', _('Password')),
-        ('L', _('List')),
-    )
-    type = models.CharField(_('Type'), max_length=1, choices=TYPES)
-    ASPECTS = (
-        ('SLOT', _('Slot')),
-        ('EVEN', _('Event')),
-        ('PREF', _('Preference')),
-        ('PROP', _('Property')),
-    )
-    secure = models.BooleanField(_('Secure'), default=False)
-    aspect = models.CharField(_('Aspect'), max_length=4, choices=ASPECTS)
-    label = models.CharField(_('Label'), max_length=150, null=False, blank=True, default='')
-    action_label = models.CharField(_('Action label'), max_length=50, null=False, blank=True, default='')
-    description = models.CharField(_('Description'), max_length=250, null=False, blank=True, default='')
-    friend_code = models.CharField(_('Friend code'), max_length=30, null=False, blank=True, default='')
-    readonly = models.BooleanField(_('Read only'), default=False)
-    default_value = models.TextField(_('Default value'), blank=True, null=True)
-    value = models.TextField(_('Value'), blank=True, null=True)
-    widget = models.ForeignKey(Widget)
-    order = models.IntegerField(default=0, blank=True)
-
-    def __str__(self):
-        return self.widget.uri + " " + self.aspect
-
-    class Meta:
-        app_label = 'platform'
-        db_table = 'wirecloud_variabledef'
-
-
-@python_2_unicode_compatible
-class UserPrefOption(TransModel):
-
-    value = models.CharField(_('Value'), max_length=50)
-    name = models.CharField(_('Name'), max_length=50)
-    variableDef = models.ForeignKey(VariableDef)
-
-    def __str__(self):
-        return self.variableDef.widget.uri + " " + self.name
-
-    class Meta:
-        app_label = 'platform'
-        db_table = 'wirecloud_userprefoption'
-
-
-@python_2_unicode_compatible
-class VariableDefAttr(models.Model):
-
-    value = models.CharField(_('Value'), max_length=30)
-    name = models.CharField(_('Name'), max_length=30)
-    variableDef = models.ForeignKey(VariableDef)
-
-    def __str__(self):
-        return self.variableDef + self.name
-
-    class Meta:
-        app_label = 'platform'
-        db_table = 'wirecloud_variabledefattr'
-
 
+@receiver(post_save, sender=CatalogueResource)
 def create_widget_on_resource_creation(sender, instance, created, raw, **kwargs):
 
     from wirecloud.catalogue import utils as catalogue
-    from wirecloud.platform.widget.utils import create_widget_from_template, create_widget_from_wgt
+    from wirecloud.platform.widget.utils import create_widget_from_wgt
 
     if not created or raw:
         return
@@ -199,11 +124,20 @@ def create_widget_on_resource_creation(sender, instance, created, raw, **kwargs)
         try:
             resource.widget
         except Widget.DoesNotExist:
-            if resource.fromWGT:
-                base_dir = catalogue.wgt_deployer.get_base_dir(resource.vendor, resource.short_name, resource.version)
-                wgt_file = WgtFile(os.path.join(base_dir, resource.template_uri))
-                resource.widget = create_widget_from_wgt(wgt_file, resource.creator)
-            else:
-                resource.widget = create_widget_from_template(resource.template_uri, resource.creator)
+            base_dir = catalogue.wgt_deployer.get_base_dir(resource.vendor, resource.short_name, resource.version)
+            wgt_file = WgtFile(os.path.join(base_dir, resource.template_uri))
+            resource.widget = create_widget_from_wgt(wgt_file, resource.creator)
 
-post_save.connect(create_widget_on_resource_creation, sender=CatalogueResource)
+        # Restore any iwidget associated with this widget
+        from wirecloud.platform.iwidget.models import IWidget
+
+        IWidget.objects.filter(widget_uri=resource.local_uri_part).update(widget=resource.widget)
+
+
+@receiver(pre_delete, sender=CatalogueResource)
+def delete_widget_on_resource_deletion(sender, instance, using, **kwargs):
+    if instance.resource_type() == 'widget':
+        try:
+            instance.widget.delete()
+        except Widget.DoesNotExist:
+            pass
